/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.Collection;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Schnittstelle fr den Zugriff auf die Berechtigungen innerhab des Rahmenwerks
 * (wird als Service zur Verfgung gestellt).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: Berechtigungen.java 45988 2013-09-13 13:30:34Z peuker $
 */
public interface Berechtigungen {

	/**
	 * Fgt eine Funktion zur Liste der Funktionen mit Berechtigungen hinzu.
	 * 
	 * @param funktion
	 *            Funktion, die hinzugefgt werden soll.
	 */
	void addOberflaechenFunktion(final FunktionMitBerechtigung funktion);

	/**
	 * Entfernt eine Funktion aus der Liste der Funktionen mit Berechtigungen.
	 * 
	 * @param id
	 *            ID der Funktion, die entfernt werden soll.
	 */
	void removeOberflaechenFunktion(final String id);

	/**
	 * liefert die registrierten Funktionen.
	 * 
	 * @return die Menge der Funktionen
	 */
	Collection<FunktionMitBerechtigung> getFunktionen();

	/**
	 * liefert den Berechtigunszustand der Funktion fr den angemeldeten Nutzer.
	 * 
	 * @param funktion
	 *            die Funktion
	 * @return der Freigabezustand
	 */
	boolean hasBerechtigung(FunktionMitBerechtigung funktion);

	/**
	 * prft, ob der als SystemObject bergebene Benutzer, die Berechtigung fr
	 * die angegebene Funktion besitzt.
	 * 
	 * @param benutzer
	 *            der Benutzer
	 * @param funktion
	 *            die Funktion
	 * @return der Zustand der Berechtigung
	 */
	boolean hasBerechtigung(SystemObject benutzer,
			FunktionMitBerechtigung funktion);

	/**
	 * prft, ob der Benutzer mit der bergebenen PID, die Berechtigung fr die
	 * angegebene Funktion besitzt.
	 * 
	 * @param benutzerPid
	 *            die PID
	 * @param funktion
	 *            die Funktion
	 * @return der Zustand der Berechtigung
	 */
	boolean hasBerechtigung(String benutzerPid, FunktionMitBerechtigung funktion);

	/**
	 * liefert die Funktion mit der angegebenen Id oder <code>null</code>, wenn
	 * keine solche existiert.
	 * 
	 * @param id
	 *            die ID
	 * @return die Funktion oder <code>null</code>
	 */
	FunktionMitBerechtigung getFunktion(String id);

	/**
	 * liefert einen Sammlung der verfgbaren Berechtigungsklassen (als
	 * SystemObject).
	 * 
	 * @return die Berechtigungsklassen
	 */
	Collection<SystemObject> getBerechtigungsKlassen();

	/**
	 * liefert einen Sammlung der verfgbaren Berechtigungsklassen fr den
	 * Nutzer mit der bergebenen PID (als SystemObject).
	 * 
	 * @param benutzerObject
	 *            das SystemObject des Nutzers, dessen Berechtigungsklassen
	 *            gesucht werden
	 * @return die Berechtigungsklassen
	 */
	Collection<SystemObject> getBerechtigungsKlassen(SystemObject benutzerObject);

	/**
	 * fgt einen Listener hinzu, der ber nderungen von Berechtigungen
	 * informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void addOberflaechenFunktionsListener(IBerechtigungListener listener);

	/**
	 * fgt einen Listener hinzu, der ber nderungen von Berechtigungen der
	 * angegebenen Funktion informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 * @param funktion
	 *            die Funktion
	 */
	void addListenerForFunktion(IBerechtigungListener listener,
			FunktionMitBerechtigung funktion);

	/**
	 * entfernt einen fr nderungen von Berechtigungen angemeldeten Listener.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void removeListener(IBerechtigungListener listener);
}
