/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.einstellungen;

import java.io.IOException;
import java.util.Map;

import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;

/**
 * Schnittstelle fr den Zugriff auf die Einstellungen des Rahmenwerks (wird als
 * Service zur Verfgung gestellt).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: Einstellungen.java 46720 2013-10-18 12:11:16Z peuker $
 */
public interface Einstellungen {

	/**
	 * liefert eine Menge der ID fr die Einstellungen mit den zugehriegn
	 * Typen, die fr den angegebenen SpeicherKey im Einstellungsspeicher
	 * verfgbar sind.
	 * 
	 * Der Schlssel in der Map entspricht der Id der jeweiligen Einstellung,
	 * der Wert entspricht dem Typ. Der Typ kann <code>null</code> sein, dann
	 * wird fr diesen der Standard-Typ java.lang.String verwendet.
	 * 
	 * @param key
	 *            der Schlssel fr den Einstellungsspeicher
	 * @return das Liste der Id
	 * @throws IOException
	 *             die Einstellung konnte nicht gelesen werden
	 */
	Map<String, String> getEinstellungsId(SpeicherKey key) throws IOException;

	/**
	 * liefert die fr die bergebene Adresse vorliegende Einstellung.
	 * 
	 * @param adresse
	 *            die Adresse
	 * @return das Einstellungsobjekt
	 * @throws IOException
	 *             die Einstellung konnte nicht gelesen werden
	 */
	Object getValue(final EinstellungsAdresse adresse) throws IOException;

	/**
	 * setzt fr die bergebene Adresse die angegebene Einstellung.
	 * 
	 * @param adresse
	 *            die Adresse
	 * @param einstellung
	 *            das Einstellungsobjekt
	 * @throws IOException
	 *             die Einstellung konnte nicht gelesen werden
	 */
	void setValue(final EinstellungsAdresse adresse, final Object einstellung)
			throws IOException;

	/**
	 * setzt fr die bergebene Adresse die angegebene Einstellung mit den
	 * bergebenen Urlasserinformationen.
	 * 
	 * @param adresse
	 *            die Adresse
	 * @param einstellung
	 *            das Einstellungsobjekt
	 * @param urlasser
	 *            die Urlasserinformationen
	 * @throws IOException
	 *             die Einstellung konnte nicht gelesen werden
	 */
	void setValue(final EinstellungsAdresse adresse, final Object einstellung,
			final UrlasserInfo urlasser) throws IOException;

	/**
	 * entfernt die unter der bergebenen Adresse hinterlegten Einstellungen.
	 * 
	 * @param adresse
	 *            die Adresse
	 */
	void removeValue(EinstellungsAdresse adresse);

	/**
	 * fgt einen Listener hinzu, der ber nderungen von Einstellungen
	 * informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void addEinstellungsListener(final EinstellungChangeListener listener);

	/**
	 * fgt einen Listener hinzu, der ber nderungen von Einstellungen der
	 * angegebenen Kategorie informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 * @param category
	 *            die Kategorie
	 */
	void addEinstellungsListener(final EinstellungChangeListener listener,
			final String category);

	/**
	 * entfernt einen Listener, der ber nderungen der Einstellungen informiert
	 * wurde.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void removeEinstellungsListener(final EinstellungChangeListener listener);

	/**
	 * entfernt einen Listener, der ber nderungen der Einstellungen in der
	 * angegebenen Kategorie informiert wurde.
	 * 
	 * @param listener
	 *            der Listener
	 * @param category
	 *            die Kategorie
	 */
	void removeEinstellungsListener(final EinstellungChangeListener listener,
			final String category);

	/**
	 * fgt einen Listener hinzu, der ber die Verfgbarkeit des
	 * Einstellungsspeichers informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void addEinstellungsAvailabilityListener(
			final EinstellungAvailabilityListener listener);

	/**
	 * entfernt einen Listener, der ber die Verfgbarkeit des
	 * Einstellungsspeichers informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void removeEinstellungsAvailabilityListener(
			final EinstellungAvailabilityListener listener);
}
