/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.legende;

import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimerTask;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;

import de.bsvrz.buv.rw.basislib.internal.RwBasisLibActivator;
import de.bsvrz.buv.rw.basislib.legende.ILegende.Corner;

/**
 * Ein schwebendes Fenster, welches als Legende fr einen Workbench Part dient.
 * 
 * <p>
 * Der Inhalt des Fensters wird mit {@link #createContents(Composite)} angelegt
 * und mit {@link #refresh()} aktualisiert.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: LegendeWindow.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class LegendeWindow extends ToolWindow {

	/** Contentprovider fr eine TreeLegende. */
	private static final class TreeLegendeContentProvider implements
			ITreeContentProvider {
		@Override
		public Object[] getElements(final Object inputElement) {
			if (inputElement != null) {
				return ((ITreeLegende) inputElement).getBausteine().toArray();
			}

			return new Object[0];
		}

		@Override
		public boolean hasChildren(final Object element) {
			return !((ILegendeBaustein) element).getBausteine().isEmpty();
		}

		@Override
		public Object getParent(final Object element) {
			return null;
		}

		@Override
		public Object[] getChildren(final Object parentElement) {
			return ((ILegendeBaustein) parentElement).getBausteine().toArray();
		}

		@Override
		public void inputChanged(final Viewer v, final Object oldInput,
				final Object newInput) {
			// tut nix
		}

		@Override
		public void dispose() {
			// tut nix
		}
	}

	/** Labelprovider fr eine TreeLegende. */
	private static final class TreeLegendeLabelProvider extends LabelProvider {
		@Override
		public String getText(final Object element) {
			return ((ILegendeBaustein) element).getText();
		}

		@Override
		public Image getImage(final Object element) {
			return ((ILegendeBaustein) element).getIcon();
		}
	}

	/** Timer zur Aktualisierung der Anzeige des letzten Datenzeitstempels. */
	private static class RefreshTimer extends TimerTask {

		private final Label displayLabel;
		private final IUpdateTimeProvider updateTimeProvider;

		public RefreshTimer(final Label displayLabel,
				final IUpdateTimeProvider updateTimeProvider) {
			this.displayLabel = displayLabel;
			this.updateTimeProvider = updateTimeProvider;
		}

		@Override
		public void run() {
			if ((displayLabel != null) && (!displayLabel.isDisposed())) {
				displayLabel.getDisplay().asyncExec(new Runnable() {
					@Override
					public void run() {
						if ((displayLabel != null)
								&& (!displayLabel.isDisposed())) {
							final String zeitStr = DateFormat
									.getDateTimeInstance().format(
											new Date(updateTimeProvider
													.getLastUpdateTime()));
							displayLabel.setText(zeitStr);
						}
					}
				});
			}
		}

	}

	/**
	 * ID des Commands zur Ein-/Ausblenden des Legendenfensters fr einen
	 * WorkbenchPart.
	 */
	public static final String COMMAND_ID_TOOGLE_LEGENDE_COMMAND_ID = "de.bsvrz.buv.rw.rw.aktion.toggle.legende";

	private static final Map<IWorkbenchPart, LegendeWindow> CACHE = new HashMap<IWorkbenchPart, LegendeWindow>();

	/**
	 * liefert das {@link LegendeWindow} fr den bergebenen WorkbenchPart aus
	 * dem Cache oder <code>null</code>, wenn keines verfgbar ist.
	 * 
	 * @param part
	 *            der WorkbenchPart
	 * @return das {@link LegendeWindow} oder <code>null</code>
	 */
	public static LegendeWindow getCachedLegendeWindowFor(
			final IWorkbenchPart part) {
		return LegendeWindow.CACHE.get(part);
	}

	private TreeViewer treeViewer;
	private RefreshTimer refreshTimer;

	{
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.TOOL);
	}

	/**
	 * Initialisiert das Legendenfenster.
	 * 
	 * @param part
	 *            der Workbench Part zu dem das Legendenfenster gehrt.
	 */
	public LegendeWindow(final IWorkbenchPart part) {
		super(part, null);
	}

	@Override
	public int open() {

		LegendeWindow.CACHE.put(getWorkbenchPart(), this);
		getParentShell().addDisposeListener(new DisposeListener() {
			@Override
			public void widgetDisposed(final DisposeEvent e) {
				LegendeWindow.CACHE.remove(getWorkbenchPart());
			}
		});
		return super.open();
	}

	@Override
	public boolean close() {

		if (refreshTimer != null) {
			refreshTimer.cancel();
			refreshTimer = null;
			RwBasisLibActivator.getPluginTimer().purge();
		}

		LegendeWindow.CACHE.remove(getWorkbenchPart());
		return super.close();
	}

	@Override
	public Control getPartControl() {
		if (getLegende() == null) {
			return null;
		}
		return getLegende().getControl();
	}

	@Override
	protected Control createContents(final Composite parent) {
		final Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout());

		final Menu menu = new Menu(container);
		for (final Corner corner : Corner.values()) {
			final MenuItem item = new MenuItem(menu, SWT.PUSH);
			item.setText(corner.getBezeichnung());
			item.setData(corner);
			item.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(final SelectionEvent e) {
					setCorner((Corner) item.getData());
					relocate(getShell());
				}
			});
		}
		container.setMenu(menu);

		final Composite mainPanel = new Composite(container, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(mainPanel);
		mainPanel.setLayout(new FillLayout());

		if (getLegende() instanceof ITreeLegende) {
			createTreeLegende(mainPanel);
			setCorner(getLegende().getDefaultCorner());
		} else if (getLegende() instanceof ICustomLegende) {
			createCustomLegende(mainPanel);
			setCorner(getLegende().getDefaultCorner());
		} else {
			final Label label = new Label(container, SWT.WRAP);
			label.setText("Keine Legende vorhanden.");
			label.setBackground(parent.getDisplay().getSystemColor(
					SWT.COLOR_WHITE));
		}

		final IUpdateTimeProvider updateTimeProvider = (IUpdateTimeProvider) getWorkbenchPart()
				.getAdapter(IUpdateTimeProvider.class);
		if (updateTimeProvider != null) {
			new Label(container, SWT.NONE).setText("Letzte Aktualisierung");
			final Label updateTimeStamp = new Label(container, SWT.CENTER);
			updateTimeStamp.setText("unbekannt");
			GridDataFactory.fillDefaults().grab(true, false)
					.applyTo(updateTimeStamp);
			refreshTimer = new RefreshTimer(updateTimeStamp, updateTimeProvider);
			RwBasisLibActivator.getPluginTimer().schedule(refreshTimer, 100,
					3000);
		}

		return container;
	}

	private void createTreeLegende(final Composite parent) {
		treeViewer = new TreeViewer(parent);
		treeViewer.setLabelProvider(new TreeLegendeLabelProvider());
		treeViewer.setContentProvider(new TreeLegendeContentProvider());
		refreshTree();
	}

	private void createCustomLegende(final Composite parent) {
		((ICustomLegende) getLegende()).createControl(parent);
	}

	private ILegende getLegende() {
		ILegende result = (ILegende) getWorkbenchPart().getAdapter(
				ILegende.class);
		if (result == null) {
			result = (ILegende) Platform.getAdapterManager().getAdapter(
					getWorkbenchPart(), ILegende.class);
		}

		return result;
	}

	@Override
	protected void configureShell(final Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Legende");
		newShell.setSize(200, 400);
	}

	/**
	 * Aktualisiert den Namen und den Inhalt des Legendenfensters. Muss bei
	 * nderungen an der Legende vom Workbench Part aufgerufen werden.
	 * 
	 * <p>
	 * Kann in abgeleiteten Klassen erweitert werden.
	 */
	public void refresh() {
		refreshTree();
	}

	private void refreshTree() {
		treeViewer.setInput(getLegende());
		treeViewer.expandAll();
	}

}
