/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Ein Objekt dieser Klasse reprsentiert einen Datensatz in der DataViewFrame
 * Online-Tabelle. Reichen die Informationen eines {@link ResultData} nicht aus,
 * so steht ein weiterer Konstruktor zur Verfgung, dem neben den bentigten
 * Werten fr die Online-Tabelle die
 * {@link de.bsvrz.dav.daf.main.archive.ArchiveDataKind Art der Archivanfrage}
 * bergeben werden kann.
 * 
 * @author Kappich Systemberatung
 * @version $Revision: 1.1 $
 */
public class DataTableObject {

	private final SystemObject object;

	private final DataDescription dataDescription;

	private final TimingType timingType;

	private final long archiveTime;

	private final long dataTime;

	private final long dataIndex;

	private final DataState dataState;

	private final ArchiveDataKind dataKind;

	private final Data data;

	/**
	 * Erstellt einen Datensatz aus den Informationen eines
	 * {@link de.bsvrz.dav.daf.main.ResultData Ergebnisdatensatzes} zur Anzeige
	 * in der DataViewFrame Online-Tabelle.
	 * 
	 * @param resultData
	 *            Ergebnisdatensatz
	 */
	public DataTableObject(final ResultData resultData) {
		this(resultData.getObject(), resultData.getDataDescription(),
				TimingType.DATA_TIME, 0, resultData.getDataTime(), resultData
						.getDataIndex(), resultData.getDataState(), (resultData
						.isDelayedData() ? ArchiveDataKind.ONLINE_DELAYED
						: ArchiveDataKind.ONLINE), resultData.getData());
	}

	/**
	 * Erstellt einen Datensatz zur Anzeige in der DataViewFrame Online-Tabelle.
	 * 
	 * @param object
	 * 
	 *            das Systemobjekt
	 * @param dataDescription
	 *            Datenbeschreibung besteht aus der Attributgruppe, Aspekt und
	 *            der Simulationsvariante
	 * @param timingType
	 *            gibt an, welcher der drei Typen (Archivzeit, Datenzeit und
	 *            Datenindex) angezeigt werden soll
	 * @param archiveTime
	 *            der Archivzeitstempel
	 * @param dataTime
	 *            der Datenzeitstempel
	 * @param dataIndex
	 *            der Datenindex
	 * @param dataState
	 *            Zustand der Daten
	 * @param dataKind
	 *            Art des Datensatzes
	 * @param data
	 *            die Daten
	 */
	public DataTableObject(final SystemObject object,
			final DataDescription dataDescription, final TimingType timingType,
			final long archiveTime, final long dataTime, final long dataIndex,
			final DataState dataState, final ArchiveDataKind dataKind,
			final Data data) {
		this.object = object;
		this.dataDescription = dataDescription;
		this.timingType = timingType;
		this.archiveTime = archiveTime;
		this.dataTime = dataTime;
		this.dataIndex = dataIndex;
		this.dataState = dataState;
		this.dataKind = dataKind;
		this.data = data;
	}

	/**
	 * Gibt das Systemobjekt zurck.
	 * 
	 * @return das Systemobjekt
	 */
	public SystemObject getObject() {
		return object;
	}

	/**
	 * Gibt die Datenbeschreibung (Attributgruppe, Aspekt und
	 * Simulationsvariante) zurck.
	 * 
	 * @return die Datenbeschreibung
	 */
	public DataDescription getDataDescription() {
		return dataDescription;
	}

	/**
	 * Gibt an, welcher Zeitstempel angezeigt werden soll.
	 * 
	 * @return die Art des Zeitstempels, der angezeigt werden soll
	 */
	public TimingType getTimingType() {
		return timingType;
	}

	/**
	 * Gibt die Archivzeit zurck.
	 * 
	 * @return die Archivzeit
	 */
	public long getArchiveTime() {
		return archiveTime;
	}

	/**
	 * Gibt die Datenzeit zurck.
	 * 
	 * @return die Datenzeit
	 */
	public long getDataTime() {
		return dataTime;
	}

	/**
	 * Gibt den Datenindex zurck.
	 * 
	 * @return der Datenindex
	 */
	public long getDataIndex() {
		return dataIndex;
	}

	/**
	 * Gibt den Zustand der Daten zurck.
	 * 
	 * @return der Zustand der Daten
	 */
	public DataState getDataState() {
		return dataState;
	}

	/**
	 * Gibt die Art des Datensatzes zurck (ob OA = online aktuell, ON = online
	 * nachgeliefert, NA = nachgefordert aktuell, NN = nachgefordert
	 * nachgeliefert - siehe auch Technische Anforderungen Archivsystem).
	 * 
	 * @return die Art des Datensatzes
	 */
	public ArchiveDataKind getDataKind() {
		return dataKind;
	}

	/**
	 * Gibt die Daten dieses Datensatzes zurck.
	 * 
	 * @return die Daten dieses Datensatzes
	 */
	public Data getData() {
		return data;
	}

	/**
	 * Gibt den Inhalt dieser Klasse als String zurck.
	 * 
	 * @return der Inhalt dieser Klasse als String
	 */
	@Override
	public String toString() {
		return "DataTableObject{" + "_object=" + object + ", _dataDescription="
				+ dataDescription + ", _timingType=" + timingType
				+ ", _archiveTime=" + archiveTime + ", _dataTime=" + dataTime
				+ ", _dataIndex=" + dataIndex + ", _dataState=" + dataState
				+ ", _dataKind=" + dataKind + ", _data=" + data + "}";
	}
}
