/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Betriebskilometer;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.dav.daf.main.ClientDavConnection;

/**
 * Filter, reprsentiert die Auswahl des Betriebskilometers.
 * 
 * @author HofmannJ
 * 
 */
public class FilterBetriebskilometer implements Cloneable {
	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection connection;

	/**
	 * Pid der selektierten Strae.
	 */
	private String pidStrasse;

	/**
	 * Selektierte {@link Richtung}.
	 */
	private Richtung richtung;

	/**
	 * Speichert den gewhlten Betriebskilometer.
	 */
	private long betriebskilometerWert = -1;

	// /**
	// * TreeMap als "Referenz" auf ListenElementObjekte vom Typ typ.strae
	// */
	// private TreeMap<String, Object> _strassen = null;

	private final LinkedList<String> blocknummern = new LinkedList<String>();

	/**
	 * Speichert die selektierte BlockNummer.
	 */
	private String selektierteBlockNummer;

	/**
	 * Speichert die bentigte Instanz der Klasse {@link Betriebskilometer}.
	 */
	private Betriebskilometer betriebsKilometer;

	/**
	 * Konstruktor. bernimmt die Verbindung zum DaV.
	 * 
	 * @param connection
	 *            Verbindung zum DaV.
	 */
	public FilterBetriebskilometer(final ClientDavConnection connection) {
		this.connection = connection;
		init();

	}

	/**
	 * Fhrt die Initialisierung der bentigten Datenstrukturen durch.
	 * 
	 */
	private void init() {
		if (connection != null) {
			betriebsKilometer = new Betriebskilometer(connection);
		}
	}

	/**
	 * Liefert die (sortierte) Liste der Straen zurck.
	 * 
	 * @return Liste aller Straen oder <code>null</code>, wenn keine Liste der
	 *         Straen bezogen werden konnte.
	 */
	public List<Strasse> getStrassen() {
		if (betriebsKilometer != null) {
			return betriebsKilometer.getStrassen();
		}
		return null;
	}

	/**
	 * Setzt die Pid der selektierten Strae.
	 * 
	 * @param pid
	 *            Pid der selektierten Strae
	 */
	public void setSelektierteStrasse(final String pid) {
		this.pidStrasse = pid;
	}

	/**
	 * Liefert die PID der selektierten Strae.
	 * 
	 * @return PID der selektierten Strae
	 */
	public String getPidSelektierteStrasse() {
		return pidStrasse;
	}

	/**
	 * Setzt die selektierte Richtung.
	 * 
	 * @param selektierteRichtung
	 *            Zu setzende Richtung
	 */
	public void setSelektierteRichtung(final Richtung selektierteRichtung) {
		this.richtung = selektierteRichtung;
	}

	/**
	 * Liefert die selektierte Richtung.
	 * 
	 * @return Selektierte Richtung
	 */
	public Richtung getSelektierteRichtung() {
		return richtung;
	}

	/**
	 * Setzt den Wert des Betriebskilometers. Gleichzeitig wird die Liste der
	 * verfgbaren Blocknummern aktualisiert. Sie ist ber die Methode
	 * {@link FilterBetriebskilometer#getVerfuegbareBlockNummern()} verfgbar.
	 * 
	 * @param betriebskilometer
	 *            Der zu setzende Wert des Betriebskilometers.
	 */
	public void setBetriebskilometer(final long betriebskilometer) {
		this.betriebskilometerWert = betriebskilometer;
	}

	/**
	 * Liefert den gesetzten Betriebskilometer Wert.
	 * 
	 * @return Gesetzter Betriebskilometer Wert.
	 */
	public long getBetriebskilometer() {
		return betriebskilometerWert;
	}

	/**
	 * Liefert die selektierte BlockNummer.
	 * 
	 * @return Selektierte BlockNummer
	 */
	public String getSelektierteBlockNummern() {
		return selektierteBlockNummer;
	}

	/**
	 * Setzt die selektierte BlockNummer.
	 * 
	 * @param blocknummer
	 *            Zu setzende, selektierte Blocknummer.
	 */
	public void setSelektierteBlockNummer(final String blocknummer) {
		selektierteBlockNummer = blocknummer;
	}

	/**
	 * Liefert eine Liste der zum gesetzten Betriebskilometer verfgbaren
	 * BlockNummern.<br>
	 * Ist die Liste <code>null</code>, liegt der gesetzte Betriebskilometer
	 * <b>nicht</b> im LCL-Segment. <br>
	 * Ist ein Eintrag <code>-1</code>, dann existiert zwar eine
	 * Betriebskilometereintrag, jedoch verfgt dieser ber <b>keine</b>
	 * Blockzuordnung.<br>
	 * 
	 * @return Liste von BlockNummern.
	 */
	public List<String> getVorhandeneBlockNummern() {

		List<String> vorhandeneBlockNummern = null;

		if ((betriebsKilometer != null) && (pidStrasse != null)
				&& (betriebskilometerWert > -1)) {
			vorhandeneBlockNummern = betriebsKilometer
					.pruefeBetriebsKilometerVorkommen(pidStrasse,
							betriebskilometerWert);

		}

		return vorhandeneBlockNummern;
	}

	@Override
	public String toString() {
		String re = "===FilterBetriebskilometer - toString===\n";
		re = re + "PID Strae:         " + pidStrasse + "\n";
		re = re + "Richtung:           " + richtung + "\n";
		re = re + "Betriebskilometer:  " + betriebskilometerWert + "\n";
		re = re + "Selektierter Block: " + selektierteBlockNummer + "\n";
		return re;
	}

	/**
	 * Enumerator fr die (Himmels-) Richtung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Richtung {
		/**
		 * Nord.
		 */
		NORD,

		/**
		 * Sd.
		 */
		SD,

		/**
		 * Ost.
		 */
		OST,

		/**
		 * West.
		 */
		WEST
	}

	@Override
	public Object clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new InternalError();
		}
		final FilterBetriebskilometer re = new FilterBetriebskilometer(
				connection);
		re.pidStrasse = pidStrasse;
		re.richtung = richtung;
		re.betriebskilometerWert = betriebskilometerWert;
		final Iterator<String> it = blocknummern.iterator();
		while (it.hasNext()) {
			re.blocknummern.add(it.next());
		}
		re.selektierteBlockNummer = selektierteBlockNummer;
		re.betriebsKilometer = betriebsKilometer;
		return re;
	}
}
