/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse die Methoden zur Verfgung stellt, die die Objekte ermittelt die fr
 * die Ermittlung der Stauprognose bentigt werden.
 * 
 * @author Dambach Werke GmbH
 */
public final class Konfiguration {

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Datenmodell.
	 */
	private DataModel dataModel;

	/**
	 * * Verbindung zum DaV.
	 */
	private ClientDavConnection davConnection;

	/**
	 * Hashmap mit Instanzen der einzelnen Staenknoten (Key = Pid, Value =
	 * Instanz).
	 */
	private final Map<String, StrassenKnoten> strassenKnotenMenge = new HashMap<String, StrassenKnoten>();

	/**
	 * Hashmap mit Instanzen der einzelnen ueren Straensegmente (Key = Pid,
	 * Value = Instanz).
	 */
	private final Map<String, AeusseresStrassenSegment> aeusseresStrassenSegment = new HashMap<String, AeusseresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen inneren Staensegmente (Key = Pid,
	 * Value = Instanz).
	 */
	private final Map<String, InneresStrassenSegment> inneresStrassenSegment = new HashMap<String, InneresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen Messquerschnitte (Key = Pid, Value =
	 * Instanz).
	 */
	private final Map<String, MessQuerschnitt> messQuerschnitt = new HashMap<String, MessQuerschnitt>();

	/**
	 * Zuordnungstabelle MessQuerschnitt zu NBA MessQuerschnitt. (Key = PID des
	 * MessQuerschnitt, Value = die Pid des NbaMessQuerschnitts).
	 */
	private static Map<String, String> zuordnungMqZuNbaMq = new HashMap<String, String>();

	/**
	 * Methode zum Lesen der einzigen Instanz der Klasse.
	 * 
	 * @return einzige Instanz der Klasse
	 */
	public static Konfiguration getInstanz() {
		return Inner.INSTANCE;
	}

	/**
	 * Innere Klasse zum Sicherstellen, dass wirklich nur eine Instanz der
	 * Klasse gebildet wird.
	 * 
	 * @author Dambach Werke GmbH
	 */
	public static class Inner {
		private static final Konfiguration INSTANCE = new Konfiguration();
	}

	/**
	 * Konstruktor der Klasse.
	 */
	private Konfiguration() {
	}

	/**
	 * Methode die das eigentliche Bestimmen der Objekte auslst. Methode
	 * bentigt die Verbindung zum DaV.
	 * 
	 * @param connection
	 *            Verbindung zum DaV
	 * @param netz
	 *            zu betrachtendes Netz
	 * @param kbMessQuerschnitte
	 *            Konfigurationsbereich(e) der zu betrachtenden Messquerschnitte
	 */
	public void bestimmeObjekte(final ClientDavConnection connection,
			final String netz, final String kbMessQuerschnitte) {

		Konfiguration.LOGGER.fine("Objekte bestimmen fr Netz: " + netz);

		davConnection = connection;

		dataModel = davConnection.getDataModel();

		Konfiguration.LOGGER.fine("Bestimme uere Straensegmente");

		bestimmeAeussereStrassenSegmente(netz);

		Konfiguration.LOGGER.fine("Bestimme Straenknoten");

		bestimmeStrassenKnoten();

		Konfiguration.LOGGER.fine("Bestimme innere Straensegmente");

		bestimmeInnereStrassenSegmente();

		Konfiguration.LOGGER.fine("Bestimme Messquerschnitte");

		bestimmeMessQuerschnitte(kbMessQuerschnitte);

		Konfiguration.LOGGER.fine("Bestimme NBA Messquerschnitte");

		bestimmeNbaMessQuerschnitte();

		ausgabeStrassenKnoten();

		ausgabeAeussereStrassenSegmente();

		ausgabeInnereStrassenSegmente();

		Konfiguration.LOGGER.fine("Anzahl uerer Strassensegmente: "
				+ aeusseresStrassenSegment.size());
		Konfiguration.LOGGER.fine("Anzahl innerer Strassensegmente: "
				+ inneresStrassenSegment.size());
		Konfiguration.LOGGER.fine("Anzahl Staenknoten            : "
				+ strassenKnotenMenge.size());
		Konfiguration.LOGGER.fine("Anzahl Messquerschnitt         : "
				+ messQuerschnitt.size());
	}

	/**
	 * Methode zum Bestimmen aller Objekte vom Typ "typ.straenKnoten".
	 */
	private void bestimmeStrassenKnoten() {
		final List<SystemObject> strassenKnoten = bestimmeObjekte("typ.straenKnoten");

		final Iterator<SystemObject> it = strassenKnoten.iterator();
		while (it.hasNext()) {
			final SystemObject objekt = it.next();

			strassenKnotenMenge.put(objekt.getPid(), new StrassenKnoten(
					davConnection, objekt));
		}
	}

	/**
	 * Methode zum Bestimmen aller Objekte com Typ "typ.ueresStraenSegment"
	 * im bergebenen Netz. Zustzlich werden die dazugehrenden Strassenknoten
	 * angelegt und eine Zuordnung der Strassensegmente zu den Strassenknoten
	 * vorgenommen.
	 */
	private void bestimmeAeussereStrassenSegmente(final String netz) {

		final ConfigurationObject co = (ConfigurationObject) dataModel
				.getObject(netz);

		if (co == null) {
			return;
		}

		final List<SystemObject> aeussereStrassenSegmente = new ArrayList<SystemObject>();
		final List<SystemObject> liste = co
				.getNonMutableSet("NetzBestandTeile").getElements();

		for (int i = 0; i < liste.size(); i++) {
			final ConfigurationObject coNetz = (ConfigurationObject) liste
					.get(i);

			if (!coNetz.getType().isOfType("typ.ueresStraenSegment")) {
				final List<SystemObject> liste2 = coNetz.getNonMutableSet(
						"NetzBestandTeile").getElements();

				for (int j = 0; j < liste2.size(); j++) {
					final ConfigurationObject coNetz2 = (ConfigurationObject) liste2
							.get(j);

					if (!"typ.ueresStraenSegment".equals(coNetz2.getType()
							.getPid())) {
						Konfiguration.LOGGER
								.fine("Kein ueresStrassenSegment gefunden!");
					} else {
						if (!aeussereStrassenSegmente.contains(coNetz2)) {
							aeussereStrassenSegmente.add(coNetz2);
						}
					}
				}
			}
		}

		// uereStraenSegmente = bestimmeObjekte( "typ.ueresStraenSegment"
		// ); // Test mit allen eren
		// Straensegmenten

		final Iterator<SystemObject> it = aeussereStrassenSegmente.iterator();
		while (it.hasNext()) {
			final SystemObject objekt = it.next();

			final AeusseresStrassenSegment strassenSegment = new AeusseresStrassenSegment(
					dataModel, objekt);
			strassenSegment.initialisiere();

			aeusseresStrassenSegment.put(objekt.getPid(), strassenSegment);

			final StrassenKnoten vonKnoten = strassenSegment.getVonKnoten();
			final StrassenKnoten nachKnoten = strassenSegment.getNachKnoten();

			if (vonKnoten != null) {
				vonKnoten
						.addAbgehendesAeusseresStrassenSegment(strassenSegment);
			}

			if (nachKnoten != null) {
				nachKnoten
						.addHinfuehrendesAeusseresStrassenSegment(strassenSegment);
			}
		}
	}

	/**
	 * Methode zum Bestimmen aller Objekte com Typ "typ.inneresStraenSegment".
	 * Zustzlich wird eine Zuordnung der Strassensegmente zu den Strassenknoten
	 * vorgenommen
	 */
	private void bestimmeInnereStrassenSegmente() {
		final List<SystemObject> innereStrassenSegmente = bestimmeObjekte("typ.inneresStraenSegment");

		final Iterator<SystemObject> it = innereStrassenSegmente.iterator();
		while (it.hasNext()) {
			final SystemObject objekt = it.next();

			final InneresStrassenSegment strassenSegment = new InneresStrassenSegment(
					dataModel, objekt);

			inneresStrassenSegment.put(objekt.getPid(), strassenSegment);

			final AeusseresStrassenSegment vonStrassenSegment = strassenSegment
					.getVonStrassenSegmemt();
			final AeusseresStrassenSegment nachStrassenSegment = strassenSegment
					.getNachStrassenSegment();

			if (((vonStrassenSegment) == null) && (nachStrassenSegment == null)) {
				inneresStrassenSegment.remove(objekt.getPid());
			} else {
				strassenSegment.initialisiere();

				if (vonStrassenSegment != null) {
					strassenSegment.setVorgaengerAufStrasse(vonStrassenSegment);

					final StrassenKnoten sk = strassenKnotenMenge
							.get(vonStrassenSegment.getNachKnoten().getPid());
					if (sk != null) {
						sk.addInneresStrassenSegment(strassenSegment);
					}

					if (vonStrassenSegment
							.liegtAufSelberStrasse(nachStrassenSegment)) {
						vonStrassenSegment
								.addNachfolgerAufStrasse(strassenSegment);
					} else {
						vonStrassenSegment.addNachfolger(strassenSegment);
					}
				}

				if (nachStrassenSegment != null) {
					strassenSegment
							.setNachfolgerAufStrasse(nachStrassenSegment);

					final StrassenKnoten sk = strassenKnotenMenge
							.get(nachStrassenSegment.getVonKnoten().getPid());
					if (sk != null) {
						sk.addInneresStrassenSegment(strassenSegment);
					}

					if (nachStrassenSegment
							.liegtAufSelberStrasse(vonStrassenSegment)) {
						nachStrassenSegment
								.addVorgaengerAufStrasse(strassenSegment);
					} else {
						nachStrassenSegment.addVorgaenger(strassenSegment);
					}
				}
			}
		}
	}

	/**
	 * Methode liefert einen Straenknoten zu einer Pid, existiert der
	 * Stasenknoten noch nicht und handlet es sich bei dem Objekt das zu der
	 * Pid gehrt um ein Objekt vom Typ 'typ.straenKnoten', so wird ein neuer
	 * StraenKnoten angelegt.
	 * 
	 * @param pid
	 *            Pid des Straenknotens
	 * @return Instanz des Straenknotens, im Fehlerfall null
	 */
	public StrassenKnoten getStrassenKnoten(final String pid) {
		if (strassenKnotenMenge.containsKey(pid)) {
			return strassenKnotenMenge.get(pid);
		}
		final ConfigurationObject objekt = (ConfigurationObject) dataModel
				.getObject(pid);
		if (objekt != null) {
			if (objekt.isOfType("typ.straenKnoten")) {
				final StrassenKnoten sk = new StrassenKnoten(davConnection,
						objekt);
				strassenKnotenMenge.put(sk.getPid(), sk);
				return sk;
			}
		}

		return null;
	}

	/**
	 * Methode liefert ein ueres Straensegment.
	 * 
	 * @param pid
	 *            Pid des Straensegments
	 * @return Instanz des Straensegments, im Fehlerfall null
	 */
	public AeusseresStrassenSegment getAeusseresStrassenSegment(final String pid) {
		if (aeusseresStrassenSegment.containsKey(pid)) {
			return aeusseresStrassenSegment.get(pid);
		}

		return null;
	}

	/**
	 * Methode liefert ein inneres Straensegment.
	 * 
	 * @param pid
	 *            Pid des Straensegments
	 * @return Instanz des Straensegments, im Fehlerfall null
	 */
	public InneresStrassenSegment getInneresStrassenSegment(final String pid) {
		if (inneresStrassenSegment.containsKey(pid)) {
			return inneresStrassenSegment.get(pid);
		}
		return null;
	}

	/**
	 * Methode bestimmt die vorhandenen Messquerschnitte und weist den einzelnen
	 * Straensegmente (innere und uere) die Messquerschnitte zu, die auf den
	 * jeweiligen Strassensegmenten liegen (geordnet nach Offset).
	 * 
	 * @param kbMessQuerschnitte
	 *            Konfigurationsbereiche der zu verwendenden Messquerschnitte
	 *            mit ":" getrennt. Leerer String: alle Konfigurationsbereiche
	 */
	private void bestimmeMessQuerschnitte(final String kbMessQuerschnitte) {
		// List<SystemObject> elemente = bestimmeObjekte(
		// "typ.messQuerschnittAllgemein" );

		final List<SystemObject> elemente = objektListeErstellen(dataModel,
				"typ.messQuerschnittAllgemein", kbMessQuerschnitte);

		for (int i = 0; i < elemente.size(); i++) {
			final ConfigurationObject co = (ConfigurationObject) elemente
					.get(i);

			final MessQuerschnitt mq = new MessQuerschnitt(davConnection, co);
			messQuerschnitt.put(co.getPid(), mq);

			final StrassenSegment linie = mq.getReferenz();

			if (linie != null) {
				linie.addMessQuerschnitt(mq);
			}
		}
	}

	/**
	 * Methode ermittelt, ob es sich bei der bergebenen Pid um die Pid eines
	 * inneren Straensegments handelt.
	 * 
	 * @param pid
	 *            zu prfende Pid
	 * @return true: Pid gehrt zu einem inneren Straensegment, sonst false
	 */
	private boolean isInneresStassenSegment(final String pid) {
		return inneresStrassenSegment.containsKey(pid);
	}

	/**
	 * Methode ermittelt, ob es sich bei der bergebenen Pid um die Pid eines
	 * ueren Straensegments handelt.
	 * 
	 * @param pid
	 *            zu prfende Pid
	 * @return true: Pid gehrt zu einem ueren Straensegment, sonst false
	 */
	private boolean isAeusseresStrassenSegment(final String pid) {
		return aeusseresStrassenSegment.containsKey(pid);
	}

	/**
	 * Methode liefert zu einem Strassensegment die dazugehrenden
	 * Messquerschnitte.
	 * 
	 * @param pidStrassenSegment
	 *            Pid des Strassensegments
	 * @return Vector mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getMessQuerschnitteVonStrassenSegment(
			final String pidStrassenSegment) {
		if (isAeusseresStrassenSegment(pidStrassenSegment)) {
			if (aeusseresStrassenSegment.containsKey(pidStrassenSegment)) {
				return aeusseresStrassenSegment.get(pidStrassenSegment)
						.getMessquerschnitte();
			}
		}

		if (isInneresStassenSegment(pidStrassenSegment)) {
			if (inneresStrassenSegment.containsKey(pidStrassenSegment)) {
				return inneresStrassenSegment.get(pidStrassenSegment)
						.getMessquerschnitte();
			}
		}

		return null;
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de).
	 * 
	 * @param objPid
	 *            Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	private List<SystemObject> bestimmeObjekte(final String objPid) {
		Konfiguration.LOGGER.fine("bestimmeObjekte --> " + objPid);

		final List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = davConnection.getDataModel().getType(objPid);

		if (typeSysObj == null) {
			return objekte;
		}

		// Elemente dieses Systemobjekts einlesen

		final List<SystemObject> typobjekte = typeSysObj.getElements();

		final Iterator<SystemObject> iteratorTypen = typobjekte.iterator();

		while (iteratorTypen.hasNext()) {
			final SystemObject sysObj = iteratorTypen.next();
			objekte.add(sysObj);
		}

		return objekte;
	}

	/**
	 * Methode zur Ausgabe der ueren Staensegmente (fr Testzwecke).
	 */
	private void ausgabeAeussereStrassenSegmente() {
		final Iterator<Entry<String, AeusseresStrassenSegment>> it = aeusseresStrassenSegment
				.entrySet().iterator();

		while (it.hasNext()) {

			final Entry<String, AeusseresStrassenSegment> entry = it.next();

			final AeusseresStrassenSegment s = entry.getValue();

			// if (!DebugFilter.debugSS( s.getPid() ))
			// continue;

			Konfiguration.LOGGER.fine("--------------------------------------");
			Konfiguration.LOGGER.fine(s.getPidRichtungStrasse());

			String buffer = null;

			long l = 0;

			buffer = "Straenteilsegmente:   ";
			final List<StrassenTeilSegment> pidsts = s
					.getStrassenTeilSegmente();
			final Iterator<StrassenTeilSegment> itvsts = pidsts.iterator();
			while (itvsts.hasNext()) {
				final StrassenTeilSegment sts = itvsts.next();
				buffer += " " + sts.getPid() + " (" + sts.getOffset() + "/"
						+ sts.getLaenge() + ")";
				l += sts.getLaenge();
			}
			Konfiguration.LOGGER.fine(buffer);

			Konfiguration.LOGGER.fine("Lnge: " + s.getLaenge() + " (" + l
					+ ")");

			buffer = "Vorgnger auf Strae:  ";
			List<StrassenSegment> pids = s.getVorgaengerAufStrasse();
			Iterator<StrassenSegment> itv = pids.iterator();
			while (itv.hasNext()) {
				final StrassenSegment s1 = itv.next();
				buffer += " " + s1.getPidRichtungStrasse() + "("
						+ s1.getLaenge() + ")";
			}
			Konfiguration.LOGGER.fine(buffer);

			if (s.getKuerzestenVorgaengerAufStrasse() != null) {
				Konfiguration.LOGGER.fine("krzester Vorgnger:   "
						+ s.getKuerzestenVorgaengerAufStrasse().getPid());
			}

			buffer = "sonstige Vorgnger:    ";
			pids = s.getVorgaenger();
			itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPidRichtungStrasse();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Nachfolger auf Strae: ";
			pids = s.getNachfolgerAufStrasse();
			itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPidRichtungStrasse();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "sonstige Nachfolger:   ";
			pids = s.getNachfolger();
			itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPidRichtungStrasse();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Einfahrten:            ";
			pids = s.getEinfahrendeStrassenSegmente();
			itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPidRichtungStrasse();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Einfahrten (MQ):       ";
			final List<MessQuerschnitt> v1 = s.getEinfahrendeMessQuerschnitte();
			final Iterator<MessQuerschnitt> itVEinfahrendeMQs = v1.iterator();
			while (itVEinfahrendeMQs.hasNext()) {
				buffer += " " + (itVEinfahrendeMQs.next()).getPid();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Ausfahrten:            ";
			pids = s.getAusfahrendeStrassenSegmente();
			itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPidRichtungStrasse();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Ausfahrten (MQ):       ";
			final List<MessQuerschnitt> v2 = s.getAusfahrendeMessQuerschnitte();
			final Iterator<MessQuerschnitt> itAusfahrendeMQs = v2.iterator();

			while (itAusfahrendeMQs.hasNext()) {
				buffer += " " + (itAusfahrendeMQs.next()).getPid();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Messquerschnitte:      ";
			final List<MessQuerschnitt> v = s.getMessquerschnitte();
			final Iterator<MessQuerschnitt> itMQs = v.iterator();
			while (itMQs.hasNext()) {
				final MessQuerschnitt m = itMQs.next();
				buffer += " " + m.getPid() + "(" + m.getOffset() + ") ";
			}
			Konfiguration.LOGGER.fine(buffer);
		}
	}

	/**
	 * Methode zur Ausgabe der inneren Staensegmente (fr Testzwecke).
	 */
	private void ausgabeInnereStrassenSegmente() {
		final Iterator<Entry<String, InneresStrassenSegment>> it = inneresStrassenSegment
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, InneresStrassenSegment> entry = it.next();

			final InneresStrassenSegment s = entry.getValue();

			// if (!DebugFilter.debugSS( s.getPid() ))
			// continue;

			Konfiguration.LOGGER.fine("--------------------------------------");

			String buffer = s.getPid();

			if (s.isEinfahrt()) {
				buffer += " (Einfahrt)";
			}

			if (s.isAusfahrt()) {
				buffer += " (Ausfahrt)";
			}

			Konfiguration.LOGGER.fine(buffer);

			long l = 0;

			buffer = "Straenteilsegmente:   ";
			final List<StrassenTeilSegment> pidsts = s
					.getStrassenTeilSegmente();
			final Iterator<StrassenTeilSegment> itvsts = pidsts.iterator();
			while (itvsts.hasNext()) {
				final StrassenTeilSegment sts = itvsts.next();
				buffer += " " + sts.getPid() + " (" + sts.getOffset() + "/"
						+ sts.getLaenge() + ")";
				l += sts.getLaenge();
			}
			Konfiguration.LOGGER.fine(buffer);

			Konfiguration.LOGGER.fine("Lnge: " + s.getLaenge() + " (" + l
					+ ")");

			buffer = "Vorgnger auf Strae:  ";
			List<StrassenSegment> pids = s.getVorgaengerAufStrasse();
			Iterator<StrassenSegment> itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPid();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Nachfolger auf Strae: ";
			pids = s.getNachfolgerAufStrasse();
			itv = pids.iterator();
			while (itv.hasNext()) {
				buffer += " " + itv.next().getPid();
			}
			Konfiguration.LOGGER.fine(buffer);

			buffer = "Messquerschnitte:      ";
			final List<MessQuerschnitt> vMQs = s.getMessquerschnitte();
			final Iterator<MessQuerschnitt> itMQs = vMQs.iterator();
			while (itMQs.hasNext()) {
				final MessQuerschnitt m = itMQs.next();
				buffer += " " + m.getPid() + "(" + m.getOffset() + ") ";
			}
			Konfiguration.LOGGER.fine(buffer);
		}
	}

	/**
	 * Methode zum Ausgeben der StassenKnoten (fr Testzwecke).
	 */
	private void ausgabeStrassenKnoten() {
		final Iterator<Entry<String, StrassenKnoten>> it = strassenKnotenMenge
				.entrySet().iterator();

		while (it.hasNext()) {
			final Entry<String, StrassenKnoten> entry = it.next();

			final StrassenKnoten sk = entry.getValue();

			Konfiguration.LOGGER.fine("--------------------------------------");
			Konfiguration.LOGGER.fine(sk.getPid() + " (" + sk.getTyp() + ")");

			Konfiguration.LOGGER.fine("Interne Strassensegmente:  ");
			final List<InneresStrassenSegment> segmente = sk
					.getInnereStrassenSegmente();

			final Iterator<InneresStrassenSegment> itv = segmente.iterator();
			while (itv.hasNext()) {
				final InneresStrassenSegment iss = itv.next();
				Konfiguration.LOGGER.fine(iss.getSystemObject().getPid());
			}
		}
	}

	/**
	 * Methode liefert ein Strassensegment zurck.
	 * 
	 * @param pid
	 *            Pid des Strassensegments
	 * @return Instanz des Strassensegments, im Fehlerfall null
	 */
	public StrassenSegment getStrassenSegment(final String pid) {
		if (isAeusseresStrassenSegment(pid)) {
			return aeusseresStrassenSegment.get(pid);
		}

		if (isInneresStassenSegment(pid)) {
			return inneresStrassenSegment.get(pid);
		}

		return null;
	}

	/**
	 * Methode ermittelt die NBA-Messquerschnitte.
	 */
	private void bestimmeNbaMessQuerschnitte() {
		final List<SystemObject> elemente = bestimmeObjekte("typ.nbaMessQuerschnitt");

		for (int i = 0; i < elemente.size(); i++) {
			final ConfigurationObject co = (ConfigurationObject) elemente
					.get(i);

			final Data confData = co.getConfigurationData(dataModel
					.getAttributeGroup("atg.nbaMessQuerschnitt"));

			if (confData == null) {
				continue;
			}

			final String ref = confData
					.getReferenceValue("MessQuerschnittReal").getValueText();

			if (ref != null) {
				// _debug.fine("Mq: " + ref + " nbaMQ: " + co.getPid());

				Konfiguration.zuordnungMqZuNbaMq.put(ref, co.getPid());
			}
		}
	}

	/**
	 * Methode bestimmt zu einem Messquerschnitt der mit der Pid vom Typ
	 * "typ.messQuerschnitt" die dazugehrende Pid vom Typ
	 * "typ.nbaMessQuerschnitt".
	 * 
	 * @param pidMq
	 *            Pid vom Typ "typ.messQuerschnitt"
	 * @return Pid vom Typ "typ.nbaMessQuerschnitt"
	 */
	public String getNbaMqVonMq(final String pidMq) {
		if (Konfiguration.zuordnungMqZuNbaMq.containsKey(pidMq)) {
			return Konfiguration.zuordnungMqZuNbaMq.get(pidMq);
		}

		return null;
	}

	/**
	 * Methode bestimmt zu einem Messquerschnitt der mit der Pid vom Typ
	 * "typ.nbaMessQuerschnitt" die dazugehrende Pid vom Typ
	 * "typ.messQuerschnitt".
	 * 
	 * @param pidNbaMq
	 *            Pid vom Typ "typ.nbaMessQuerschnitt"
	 * @return Pid vom Typ "typ.messQuerschnitt"
	 */
	public String getMqVonNbaMq(final String pidNbaMq) {
		final Iterator<Entry<String, String>> it = Konfiguration.zuordnungMqZuNbaMq
				.entrySet().iterator();

		while (it.hasNext()) {
			final Entry<String, String> entry = it.next();

			final String entryPidMq = entry.getKey();
			final String entryPidNbaMq = entry.getValue();

			if (pidNbaMq.equals(entryPidNbaMq)) {
				return entryPidMq;
			}
		}

		return null;
	}

	/**
	 * Methode liefert alle Messquerschnitte (ty.messQuerschnitt).
	 * 
	 * @return Liste mit den Messquerschnitten
	 */
	public List<SystemObject> getMessQuerschnitte() {
		final List<SystemObject> l = new ArrayList<SystemObject>();

		final Iterator<Entry<String, MessQuerschnitt>> it = messQuerschnitt
				.entrySet().iterator();

		while (it.hasNext()) {
			final Entry<String, MessQuerschnitt> entry = it.next();

			final MessQuerschnitt mq = entry.getValue();

			l.add(mq.getSystemObjekt());
		}

		return l;
	}

	/**
	 * Methode zur Erstellung einer Liste von Objekten bestimmter
	 * Konfigurationsbereiche. Es knnen die Pids der Konfigurationsbereichen
	 * getrennt durch ':' bergeben werden.
	 * 
	 * @param usedDataModel
	 *            Datenmodell
	 * @param pidObjektTyp
	 *            Pid des gesuchten ObjektTyps
	 * @param konfigurationsBereiche
	 *            Pids der Konfigurationsbereiche getrennt durch ';'.
	 * @throws IllegalArgumentException
	 */
	public List<SystemObject> objektListeErstellen(
			final DataModel usedDataModel, final String pidObjektTyp,
			final String konfigurationsBereiche) {

		if (usedDataModel.getType(pidObjektTyp) == null) {
			throw new IllegalArgumentException(
					"Gesuchter Objekttyp nicht vorhanden " + pidObjektTyp);
		}
		final List<SystemObject> objekte = new ArrayList<SystemObject>();

		String[] argumente = null;

		List<SystemObject> alleObjekteTyp = null;

		// keine Argumente bergeben, dann alle Konfigurationsbereiche verwenden

		if (konfigurationsBereiche.length() == 0) {

			Konfiguration.LOGGER
					.fine("Kein Argument bergeben, alle Konfigurationsbereiche werden verwendet.");

			final SystemObjectType sotKonfigurationsbereiche = usedDataModel
					.getType("typ.konfigurationsBereich");

			final List<SystemObject> listKonfigurationsbereiche = sotKonfigurationsbereiche
					.getElements();

			argumente = new String[listKonfigurationsbereiche.size()];

			for (int i = 0; i < listKonfigurationsbereiche.size(); i++) {
				final ConfigurationObject konfigurationsbereich = (ConfigurationObject) listKonfigurationsbereiche
						.get(i);

				argumente[i] = konfigurationsbereich.getPid();
			}
		} else {
			// vorhandene Argumente bernehmen
			if (konfigurationsBereiche.length() != 0) {

				Konfiguration.LOGGER.fine("Argument '" + konfigurationsBereiche
						+ " wird verwendet.");

				argumente = konfigurationsBereiche.split(":");
			}
		}

		// Einzelobjekte bestimmen

		if (argumente != null) {
			for (final String element : argumente) {
				Konfiguration.LOGGER.fine("Konfiguratiosbereich: " + element);

				final SystemObject so = usedDataModel.getObject(element);

				if (so != null) {
					if (so.isOfType(pidObjektTyp)) {
						Konfiguration.LOGGER.finer("Einzelobjekt gefunden "
								+ so.getPid());

						if (!objekte.contains(so)) {
							objekte.add(so);

							Konfiguration.LOGGER
									.finest("MessQuerschnitt in Liste eingefgt "
											+ so.getPid());
						} else {
							Konfiguration.LOGGER
									.finest("MessQuerschnitt schon in Liste vorhanden "
											+ so.getPid());
						}
					} else {
						if (so.isOfType("typ.konfigurationsBereich")) {
							Konfiguration.LOGGER
									.finer("Konfigurationsbereich gefunden "
											+ so.getPid());

							if (alleObjekteTyp == null) {
								alleObjekteTyp = usedDataModel.getType(
										pidObjektTyp).getElements();
							}

							for (int j = 0; j < alleObjekteTyp.size(); j++) {
								final SystemObject so1 = alleObjekteTyp.get(j);

								if (!so1.getConfigurationArea().equals(so)) {
									continue;
								}

								if (!objekte.contains(so1)) {
									objekte.add(so1);

									Konfiguration.LOGGER
											.finest("MessQuerschnitt in Liste eingefgt "
													+ so1.getPid()
													+ " "
													+ so1.getConfigurationArea()
															.getPid());
								} else {
									Konfiguration.LOGGER
											.finest("MessQuerschnitt schon in Liste vorhanden "
													+ so1.getPid());
								}
							}
						} else {
							// Fehlermeldung, falscher Typ

							Konfiguration.LOGGER
									.info("Falscher Typ gefunden "
											+ so.getPid() + " "
											+ so.getType().getPid());
						}
					}
				} else {
					// Fehlermeldung, Objekt nicht in Konfiguration

					Konfiguration.LOGGER.info("Objekt nicht in Konfiguration "
							+ element);
				}
			}
		}

		return objekte;
	}

	/**
	 * Methode liefert alle ueren Straensegmente.
	 * 
	 * @return Vektor mit eren Straensegmenten
	 */
	public List<AeusseresStrassenSegment> getAeussereStrassenSegmente() {
		final List<AeusseresStrassenSegment> v = new ArrayList<AeusseresStrassenSegment>();

		final Iterator<Entry<String, AeusseresStrassenSegment>> it = aeusseresStrassenSegment
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, AeusseresStrassenSegment> entry = it.next();

			final AeusseresStrassenSegment s = entry.getValue();

			v.add(s);
		}

		return v;
	}

	/**
	 * Methode liefert alle inneren Straensegmente.
	 * 
	 * @return Vektor mit inneren Straensegmenten
	 */
	public List<InneresStrassenSegment> getInnereStrassenSegmente() {
		final List<InneresStrassenSegment> v = new ArrayList<InneresStrassenSegment>();

		final Iterator<Entry<String, InneresStrassenSegment>> it = inneresStrassenSegment
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, InneresStrassenSegment> entry = it.next();

			final InneresStrassenSegment s = entry.getValue();

			v.add(s);
		}

		return v;
	}

	/**
	 * Methode liefert alle Straenteilsegmente.
	 * 
	 * @return Straensegmente
	 */
	public List<StrassenTeilSegment> getStrassenTeilSegmente() {
		final List<StrassenTeilSegment> v = new ArrayList<StrassenTeilSegment>();

		// uere Straensegmente

		final Iterator<AeusseresStrassenSegment> itAussen = getAeussereStrassenSegmente()
				.iterator();
		while (itAussen.hasNext()) {
			final AeusseresStrassenSegment s = itAussen.next();

			final Iterator<StrassenTeilSegment> itSts = s
					.getStrassenTeilSegmente().iterator();
			while (itSts.hasNext()) {
				v.add(itSts.next());
			}
		}

		// innere Straensegmente

		final Iterator<InneresStrassenSegment> itI = getInnereStrassenSegmente()
				.iterator();
		while (itI.hasNext()) {
			final InneresStrassenSegment s = itI.next();

			final Iterator<StrassenTeilSegment> itSts = s
					.getStrassenTeilSegmente().iterator();
			while (itSts.hasNext()) {
				v.add(itSts.next());
			}
		}

		return v;
	}
}
