/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.basislib.util;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.basislib.internal.RwBasisLibActivator;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Gespeicherte Dialogposition.
 *
 * Dialogpositionen knnen unter einem Name in den Profrenzen des Plug-ins
 * "Basislib" abgelegt und wieder abgerufen werden.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: SavedDialogPosition.java 50156 2014-04-14 12:25:28Z peuker $
 */
public class SavedDialogPosition implements DialogPosition {

	/** Logger fr Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	private final int xPos;
	private final int yPos;
	private final int width;
	private final int height;
	private final DialogPosition defaultPosition;

	/**
	 * speichert die aktuelle Poision des angegebenen Fensters unter dem
	 * angegebenen Name in den Profrenzen des Plug-ins.
	 *
	 * @param window
	 *            das Fenster
	 * @param name
	 *            der Name unter dem die Position abgespeichert werden soll.
	 */
	public static void savePosition(final Window window, final String name) {
		final IPreferenceStore store = RwBasisLibActivator.getDefault()
				.getPreferenceStore();

		final Shell shell = window.getShell();
		if (shell == null) {
			SavedDialogPosition.LOGGER
					.warning("Kein Shell fr das zu speichernde Window: \""
							+ window + "\" ermittelbar!");
			return;
		}

		store.setValue("SavedDialogPosition." + name + ".X",
				shell.getLocation().x);
		store.setValue("SavedDialogPosition." + name + ".Y",
				shell.getLocation().y);
		store.setValue("SavedDialogPosition." + name + ".W", shell.getSize().x);
		store.setValue("SavedDialogPosition." + name + ".H", shell.getSize().y);
	}

	/**
	 * Konstruktor, erzeugt eine Position, deren Koordinaten unter dem
	 * bergebenen Name abgelegt wurden. Falls keine Koordinaten unter dem
	 * angegebenen Name abgelegt bzw. diese entfernt wurden, wird die bergebene
	 * Standardposition verwendet.
	 *
	 * @param name
	 *            der Name unter dem eine Position abgelegt wurde.
	 * @param defaultPosition
	 *            die Standardposition, wenn keine gespeicherte vorliegt
	 */
	public SavedDialogPosition(final String name,
			final DialogPosition defaultPosition) {
		this.defaultPosition = defaultPosition;
		final IPreferenceStore store = RwBasisLibActivator.getDefault()
				.getPreferenceStore();

		store.setDefault("SavedDialogPosition." + name + ".X", -1);
		store.setDefault("SavedDialogPosition." + name + ".Y", -1);
		xPos = store.getInt("SavedDialogPosition." + name + ".X");
		yPos = store.getInt("SavedDialogPosition." + name + ".Y");
		height = store.getInt("SavedDialogPosition." + name + ".H");
		width = store.getInt("SavedDialogPosition." + name + ".W");
	}

	/**
	 * Konstruktor, erzeugt eine Position, deren Koordinaten unter dem
	 * bergebenen Name abgelegt wurden. Falls keine Koordinaten unter dem
	 * angegebenen Name abgelegt bzw. diese entfernt wurden, wird die
	 * Standardposition {@link DefaultDialogPosition#APP_CENTER} verwendet.
	 *
	 * @param name
	 *            der Name unter dem die Position gespeichert wurde.
	 */
	public SavedDialogPosition(final String name) {
		this(name, DefaultDialogPosition.APP_CENTER);
	}

	@Override
	public Point getLocation(final Window window, final Point size) {
		if ((xPos >= 0) && (yPos >= 0)) {
			return new Point(xPos, yPos);
		}
		return defaultPosition.getLocation(window, size);
	}

	/**
	 * liefert die gespeicherte Gre fr das Window.
	 *
	 * @param defaultSize
	 *            die Gre, wenn keine gespeichert wurde
	 * @return die Gre
	 */
	public Point getSavedSize(final Point defaultSize) {
		if ((width > 0) && (height > 0)) {
			return new Point(width, height);
		}

		return defaultSize;
	}
}
