package de.bsvrz.buv.rw.rw.berechtigungen;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.DynamicObjectType.DynamicObjectCreatedListener;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Interne Verwaltung der Benutzerklassen und Benutzer zur Zuordnung der
 * nutzerklassenbezogenen Berechtigung zu Nutzern.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BenutzerVerwaltung.java 50721 2014-06-19 09:03:44Z peuker $
 */
class BenutzerVerwaltung {

	/** Logger fr Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * interner Listener zur Erfassung von nderungen der Nutzer im angebundenen
	 * Datenverteilersystem.
	 */
	private class BenutzerListener implements DynamicObjectCreatedListener,
			InvalidationListener, ClientReceiverInterface {

		@Override
		public void objectCreated(final DynamicObject object) {
			addBenutzer(object);
		}

		@Override
		public void invalidObject(final DynamicObject object) {
			removeBenutzer(object);
		}

		@Override
		public void update(final ResultData[] results) {
			for (final ResultData result : results) {
				updateBenutzer((DynamicObject) result.getObject(),
						result.getData());
			}
		}
	}

	private ClientDavInterface dav;
	private final Map<String, Set<SystemObject>> berechtigungsKlassenProBenutzerPid = new HashMap<String, Set<SystemObject>>();
	private boolean usesBerechtigungenNeu;
	private final BenutzerListener benutzerListener = new BenutzerListener();
	private DataDescription benutzerDataDesc;

	private final Map<String, SystemObject> berechtigungsKlassen = new HashMap<String, SystemObject>();

	public void init(final Rahmenwerk rahmenwerk) {
		if (rahmenwerk.getDavVerbindung() == null) {
			reset();
		} else {
			this.dav = rahmenwerk.getDavVerbindung();
			usesBerechtigungenNeu = rahmenwerk.usesBerechtigungenNeu();

			berechtigungsKlassen.clear();

			AttributeGroup attributeGroup = null;
			if (usesBerechtigungenNeu) {
				for (final SystemObject object : dav.getDataModel()
						.getType("typ.berechtigungsklasseNeu").getElements()) {
					berechtigungsKlassen.put(object.getPid(), object);
				}
				attributeGroup = dav.getDataModel().getAttributeGroup(
						"atg.berechtigungsklassen");
			} else {
				for (final SystemObject object : dav.getDataModel()
						.getType("typ.berechtigungsklasse").getElements()) {
					berechtigungsKlassen.put(object.getPid(), object);
				}
				attributeGroup = dav.getDataModel().getAttributeGroup(
						"atg.benutzerParameter");
			}
			benutzerDataDesc = new DataDescription(attributeGroup, dav
					.getDataModel().getAspect("asp.parameterSoll"));

			final DynamicObjectType benutzerTyp = (DynamicObjectType) dav
					.getDataModel().getType("typ.benutzer");
			for (final SystemObject benutzer : benutzerTyp.getElements()) {
				addBenutzer((DynamicObject) benutzer);
			}

			benutzerTyp.addObjectCreationListener(benutzerListener);
			benutzerTyp.addInvalidationListener(benutzerListener);
		}
	}

	private void addBenutzer(final DynamicObject benutzer) {

		synchronized (berechtigungsKlassenProBenutzerPid) {
			Set<SystemObject> klassen = berechtigungsKlassenProBenutzerPid
					.get(benutzer.getPid());
			if (klassen == null) {
				klassen = new HashSet<SystemObject>();
				dav.subscribeReceiver(benutzerListener, benutzer,
						benutzerDataDesc, ReceiveOptions.normal(),
						ReceiverRole.receiver());
				berechtigungsKlassenProBenutzerPid.put(benutzer.getPid(),
						klassen);
			}
		}
	}

	private void removeBenutzer(final DynamicObject benutzer) {

		synchronized (berechtigungsKlassenProBenutzerPid) {
			final Set<SystemObject> klassen = berechtigungsKlassenProBenutzerPid
					.remove(benutzer.getPid());
			if (klassen != null) {
				dav.unsubscribeReceiver(benutzerListener, benutzer,
						benutzerDataDesc);
			}
		}
	}

	public void updateBenutzer(final DynamicObject benutzer, final Data data) {

		synchronized (berechtigungsKlassenProBenutzerPid) {
			final Set<SystemObject> klassen = berechtigungsKlassenProBenutzerPid
					.get(benutzer.getPid());
			if (klassen != null) {
				klassen.clear();
				if (data != null) {
					if (usesBerechtigungenNeu) {
						for (final Data.ReferenceValue reference : data
								.getReferenceArray("Berechtigungsklassen")
								.getReferenceValues()) {
							klassen.add(reference.getSystemObject());
							addBerechtigungsKlasse(benutzer, klassen,
									reference.getId());
						}
					} else {
						addBerechtigungsKlasse(benutzer, klassen, data
								.getReferenceValue("berechtigungsklasse")
								.getId());
					}
				}
			}
		}
	}

	private void addBerechtigungsKlasse(final DynamicObject benutzer,
			final Set<SystemObject> klassen, final long id) {
		if (id > 0) {
			final SystemObject klassenObj = benutzer.getDataModel().getObject(
					id);
			if (klassenObj == null) {
				LOGGER.warning("Die Berechtigungsklasse des Benutzers \""
						+ benutzer.getNameOrPidOrId() + "\" mit der ID " + id
						+ " konnte nicht ermittelt werden!");
			} else {
				klassen.add(klassenObj);
			}
		}
	}

	public void reset() {
		if (dav != null) {
			synchronized (berechtigungsKlassenProBenutzerPid) {
				berechtigungsKlassenProBenutzerPid.clear();
			}
		}

		dav = null;
		benutzerDataDesc = null;
	}

	public Collection<SystemObject> getBerechtigungsKlassen(
			final String benutzerPid) {

		synchronized (berechtigungsKlassenProBenutzerPid) {
			final Set<SystemObject> klassen = berechtigungsKlassenProBenutzerPid
					.get(benutzerPid);
			if (klassen != null) {
				return Collections.unmodifiableCollection(klassen);
			}
		}
		return Collections.emptyList();
	}

	public Collection<SystemObject> getBerechtigungsKlassen() {
		return Collections
				.unmodifiableCollection(berechtigungsKlassen.values());
	}

	public SystemObject getBerechtigungsKlasse(final String klassenPid) {
		return berechtigungsKlassen.get(klassenPid);
	}
}
