package de.bsvrz.buv.rw.rw.berechtigungen;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.berechtigung.BerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.Berechtigungen;
import de.bsvrz.buv.rw.basislib.berechtigung.FunktionMitBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.IBerechtigungListener;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungsKey;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungsKlasse;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.Funktion;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.PluginMitBerechtigungen;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Komponente zur Implementierung des "Berechtigungsservice", wird als
 * OSGI-Service in der Applikation zur Verfgung gestellt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungenImpl.java 49030 2014-02-28 15:19:54Z peuker $
 */
public final class BerechtigungenImpl implements Berechtigungen {

	/** interner Task zur Aktualisierung der Berechtigungen in der Verwaltung. */
	private class BerechtigungsAktualisierer extends Thread implements
			ClientReceiverInterface {

		private final List<Data> requestListe = new ArrayList<Data>();
		private final Object mutex = new Object();

		public BerechtigungsAktualisierer() {
			super("BerechtigungsAktualisierer");
			setDaemon(true);
		}

		@Override
		public void run() {
			while (true) {
				Data data = null;
				boolean execute = false;
				synchronized (requestListe) {
					if (!requestListe.isEmpty()) {
						data = requestListe.remove(0);
						execute = true;
					}
				}

				if (execute) {
					aktualisiereBerechtigungen(data);
				} else {
					synchronized (mutex) {
						try {
							mutex.wait(30000);
						} catch (final InterruptedException e) {
							BerechtigungenImpl.LOGGER.warning(e
									.getLocalizedMessage());
						}
					}
				}
			}
		}

		@Override
		public void update(final ResultData[] results) {
			synchronized (requestListe) {
				for (final ResultData result : results) {
					requestListe.add(result.getData());
				}
			}
			synchronized (mutex) {
				mutex.notifyAll();
			}
		}
	}

	/** Manager zur Verwaltung der Listener fr nderungen von Berechtigungen. */
	private static class BerechtigungsListenerManager {

		private final Collection<IBerechtigungListener> allgemeinListeners = new ArrayList<IBerechtigungListener>();
		private final Map<String, Collection<IBerechtigungListener>> funktionListeners = new HashMap<String, Collection<IBerechtigungListener>>();

		public void add(final IBerechtigungListener listener) {
			synchronized (allgemeinListeners) {
				allgemeinListeners.add(listener);
			}
		}

		public void add(final IBerechtigungListener listener,
				final FunktionMitBerechtigung funktion) {
			synchronized (funktionListeners) {
				Collection<IBerechtigungListener> listeners = funktionListeners
						.get(funktion.getId());
				if (listeners == null) {
					listeners = new ArrayList<IBerechtigungListener>();
					funktionListeners.put(funktion.getId(), listeners);
				}
				listeners.add(listener);
			}
		}

		public Collection<IBerechtigungListener> getListeners(
				final String funktion) {

			final Collection<IBerechtigungListener> result = new HashSet<IBerechtigungListener>();
			synchronized (allgemeinListeners) {
				result.addAll(allgemeinListeners);
			}

			synchronized (funktionListeners) {
				final Collection<IBerechtigungListener> listeners = funktionListeners
						.get(funktion);
				if (listeners != null) {
					result.addAll(listeners);
				}
			}

			return result;
		}

		public void remove(final IBerechtigungListener listener) {
			synchronized (allgemeinListeners) {
				allgemeinListeners.remove(listener);
			}

			synchronized (funktionListeners) {
				final Set<String> cleanable = new HashSet<String>();
				for (final Entry<String, Collection<IBerechtigungListener>> item : funktionListeners
						.entrySet()) {
					item.getValue().remove(listener);
					if (item.getValue().isEmpty()) {
						cleanable.add(item.getKey());
					}
				}
				for (final String id : cleanable) {
					funktionListeners.remove(id);
				}
			}
		}

	}

	/**
	 * liefert die globale Instanz.
	 * 
	 * @return die Instanz oder <code>null</code>, wenn die
	 *         Berechtigungsverwaltung noch nicht aktiviert wurde
	 */
	public static BerechtigungenImpl getInstance() {
		return BerechtigungenImpl.instance;
	}

	private static final Debug LOGGER = Debug.getLogger();
	private final Map<String, FunktionMitBerechtigung> funktionen = new HashMap<String, FunktionMitBerechtigung>();
	// private final Map<String, Set<String>> berechtigungen = new
	// HashMap<String, Set<String>>();
	private final BenutzerVerwaltung benutzerVerwaltung = new BenutzerVerwaltung();

	private final PluginMitBerechtigungen nichtZugeordneteFunktionen = new PluginMitBerechtigungen(
			"Plugin nicht bekannt");
	private final Map<String, PluginMitBerechtigungen> plugins = new HashMap<String, PluginMitBerechtigungen>();

	private Rahmenwerk rahmenwerk;

	private final BerechtigungsAktualisierer aktualisierer = new BerechtigungsAktualisierer();
	private final BerechtigungsListenerManager listenerManager = new BerechtigungsListenerManager();
	private boolean berechtigungsKonzeptAktiv = true;

	/** der zuletzt empfangene Berechtigungsdatensatz. */
	private Data berechtigungsDatenSatz;

	/** Property zur Benachrichtigung bei einem neuen Parameterdatensatz. */
	public static final String PROP_DATENSATZ = "datensatz";

	private final PropertyChangeSupport propSupport = new PropertyChangeSupport(
			this);

	private static BerechtigungenImpl instance;

	private BerechtigungsParameterDatensatz berechtigungsParameter = new BerechtigungsParameterDatensatz(
			this, null);
	private final Object parameterMutex = new Object();

	/**
	 * Konstruktor, initialisiert die Berechtigungsverwaltungen mit den
	 * Funktionen die ber den Extension-Point
	 * "de.bsvrz.buv.rw.basislib.funktionmitberechtigung" definiert werden.
	 */
	public BerechtigungenImpl() {
		loadExtensions();
		BerechtigungenImpl.instance = this;
	}

	/** aktiviert die Komponente. */
	protected void activate() {

		final ArgumentList startParameter = rahmenwerk.getStartParameter();
		if (startParameter.hasArgument("-berechtigungsKonzept")) {
			berechtigungsKonzeptAktiv = startParameter.fetchArgument(
					"-berechtigungsKonzept").booleanValue();
		}

		aktualisierer.start();

		rahmenwerk.addDavVerbindungsListener(new DavVerbindungsListener() {
			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				reset();
			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				return false;
			}

			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				init(event.getDavVerbindung());
			}
		});
		if (rahmenwerk.getDavVerbindung() != null) {
			init(rahmenwerk.getDavVerbindung());
		}
	}

	/**
	 * fgt einen Listener hinzu, der ber einen neuen Parameterdatensatz
	 * informiert werden soll.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addDatenSatzListener(final PropertyChangeListener listener) {
		propSupport.addPropertyChangeListener(listener);
	}

	@Override
	public void addListenerForFunktion(final IBerechtigungListener listener,
			final FunktionMitBerechtigung funktion) {
		listenerManager.add(listener, funktion);
	}

	@Override
	public void addOberflaechenFunktion(final FunktionMitBerechtigung funktion) {
		synchronized (funktionen) {
			final FunktionMitBerechtigung previousFunction = funktionen.put(
					funktion.getId(), funktion);
			if (previousFunction != null) {
				nichtZugeordneteFunktionen.removeFunktion(previousFunction
						.getId());
				for (final PluginMitBerechtigungen plugin : plugins.values()) {
					plugin.removeFunktion(previousFunction.getId());
				}
				BerechtigungenImpl.LOGGER.warning("Die mit der ID "
						+ funktion.getId()
						+ " definierte Funktion ersetzt die bisherige "
						+ previousFunction);
			}

			final String pluginId = funktion.getPluginId();
			if (pluginId == null) {
				nichtZugeordneteFunktionen.addFunktion(new Funktion(funktion));
			} else {
				PluginMitBerechtigungen plugin = plugins.get(pluginId);
				if (plugin == null) {
					plugin = new PluginMitBerechtigungen(pluginId);
					plugins.put(pluginId, plugin);
				}
				plugin.addFunktion(new Funktion(funktion));
			}

			for (final SystemObject klasse : getBerechtigungsKlassen()) {
				final boolean freigegeben = hasKlasseBerechtigung(klasse,
						funktion);
				fireBerechtigungsListeners(funktion.getId(), klasse.getPid(),
						freigegeben);
			}
		}
	}

	@Override
	public void addOberflaechenFunktionsListener(
			final IBerechtigungListener listener) {
		listenerManager.add(listener);
	}

	private void aktualisiereBerechtigungen(final Data data) {
		if (berechtigungsKonzeptAktiv) {
			triggerEntferneAlleBerechtigungen();
		}

		synchronized (parameterMutex) {
			berechtigungsParameter = new BerechtigungsParameterDatensatz(this,
					data);
		}

		if (berechtigungsKonzeptAktiv) {
			triggerAktualisiereAlleBerechtigungen();
		}

		final Data oldBerechtigungsDatenSatz = berechtigungsDatenSatz;
		berechtigungsDatenSatz = data;
		propSupport.firePropertyChange(BerechtigungenImpl.PROP_DATENSATZ,
				oldBerechtigungsDatenSatz, berechtigungsDatenSatz);
	}

	/**
	 * bindet eine Instanz des Service "Rahmenwerk" an die Komponente.
	 * 
	 * @param newRahmenwerk
	 *            der Service
	 */
	protected void bindRahmenwerk(final Rahmenwerk newRahmenwerk) {
		this.rahmenwerk = newRahmenwerk;
	}

	private void triggerEntferneAlleBerechtigungen() {
		synchronized (funktionen) {
			for (final SystemObject klasse : getBerechtigungsKlassen()) {
				for (final FunktionMitBerechtigung funktion : funktionen
						.values()) {
					fireBerechtigungsListeners(funktion.getId(),
							klasse.getPid(), false);
				}
			}
		}
	}

	private void triggerAktualisiereAlleBerechtigungen() {
		synchronized (funktionen) {
			for (final SystemObject klasse : getBerechtigungsKlassen()) {
				for (final FunktionMitBerechtigung funktion : funktionen
						.values()) {

					boolean freigegeben;
					synchronized (parameterMutex) {
						freigegeben = berechtigungsParameter
								.isFreigegeben(new BerechtigungsKey(
										new BerechtigungsKlasse(klasse),
										new Funktion(funktion)));
					}
					fireBerechtigungsListeners(funktion.getId(),
							klasse.getPid(), freigegeben);
				}
			}
		}
	}

	private void fireBerechtigungsListeners(final String funktion,
			final String klasse, final boolean freigegeben) {

		final SystemObject benutzer = rahmenwerk.getBenutzer();
		if (benutzer != null) {
			final SystemObject klassenObjekt = benutzerVerwaltung
					.getBerechtigungsKlasse(klasse);
			final Collection<SystemObject> berechtigungsKlassen = benutzerVerwaltung
					.getBerechtigungsKlassen(benutzer.getPid());
			if (berechtigungsKlassen.contains(klassenObjekt)) {
				final FunktionMitBerechtigung funktionsObjekt = funktionen
						.get(funktion);

				if ((klassenObjekt != null) && (funktionsObjekt != null)) {
					final BerechtigungEreignis ereignis = new BerechtigungEreignis(
							this, freigegeben, klassenObjekt,
							Collections.singleton(funktionsObjekt));
					for (final IBerechtigungListener listener : listenerManager
							.getListeners(funktion)) {
						if (freigegeben) {
							listener.freigabe(ereignis);
						} else {
							listener.sperrung(ereignis);
						}
					}
				}
			}
		}
	}

	/**
	 * liefert den zuletzt empfangenen Berechtigungsdatensatz.
	 * 
	 * @return den Datensatz
	 */
	public Data getBerechtigungsDatenSatz() {
		return berechtigungsDatenSatz;
	}

	@Override
	public Collection<SystemObject> getBerechtigungsKlassen() {
		return benutzerVerwaltung.getBerechtigungsKlassen();
	}

	@Override
	public Collection<SystemObject> getBerechtigungsKlassen(
			final SystemObject userObject) {
		if (userObject == null) {
			return Collections.emptyList();
		}

		return benutzerVerwaltung.getBerechtigungsKlassen(userObject.getPid());
	}

	@Override
	public FunktionMitBerechtigung getFunktion(final String id) {
		synchronized (funktionen) {
			return funktionen.get(id);
		}
	}

	@Override
	public Collection<FunktionMitBerechtigung> getFunktionen() {
		synchronized (funktionen) {
			return Collections.unmodifiableCollection(funktionen.values());
		}
	}

	/**
	 * liefert die Liste der Funktionen, die nicht einem Plugin zugeordnet
	 * werden konnten.
	 * 
	 * @return die liste
	 */
	public PluginMitBerechtigungen getNichtZugeordneteFunktionen() {
		return nichtZugeordneteFunktionen;
	}

	/**
	 * liefert die Liste der Plugins, die Funktionen bereitstellen.
	 * 
	 * @return die Liste
	 */
	public Collection<PluginMitBerechtigungen> getPlugins() {
		return Collections.unmodifiableCollection(plugins.values());
	}

	/**
	 * liefert den Rahemnwerkservice der Berechtigungsverwaltung oder
	 * <code>null</code>, wenn der Service nicht nicht gebunden wurde.
	 * 
	 * @return den Service oder <code>null</code>
	 */
	public Rahmenwerk getRahmenwerk() {
		return rahmenwerk;
	}

	@Override
	public boolean hasBerechtigung(final FunktionMitBerechtigung funktion) {

		boolean result = false;

		if (funktion == null) {
			return false;
		}

		String benutzerPid = null;

		if (rahmenwerk.isOnline()) {
			benutzerPid = rahmenwerk.getBenutzer().getPid();
		}

		if (berechtigungsKonzeptAktiv) {
			if (benutzerPid != null) {
				for (final SystemObject klasse : benutzerVerwaltung
						.getBerechtigungsKlassen(benutzerPid)) {
					synchronized (parameterMutex) {
						if (berechtigungsParameter
								.isFreigegeben(new BerechtigungsKey(
										new BerechtigungsKlasse(klasse),
										new Funktion(funktion)))) {
							result = true;
							break;
						}
					}
				}
			}
		} else {
			result = true;
		}

		return result;
	}

	@Override
	public boolean hasBerechtigung(final String benutzerPid,
			final FunktionMitBerechtigung funktion) {

		boolean result = false;

		if (berechtigungsKonzeptAktiv) {
			if (benutzerPid != null) {

				final SystemObject berechtigungsKlasse = benutzerVerwaltung
						.getBerechtigungsKlasse(benutzerPid);
				if (berechtigungsKlasse == null) {
					for (final SystemObject klasse : benutzerVerwaltung
							.getBerechtigungsKlassen(benutzerPid)) {
						synchronized (parameterMutex) {
							if (berechtigungsParameter
									.isFreigegeben(new BerechtigungsKey(
											new BerechtigungsKlasse(klasse),
											new Funktion(funktion)))) {
								result = true;
								break;
							}
						}
					}
				} else {
					result = hasKlasseBerechtigung(berechtigungsKlasse,
							funktion);
				}
			}
		} else {
			result = true;
		}

		return result;
	}

	@Override
	public boolean hasBerechtigung(final SystemObject benutzer,
			final FunktionMitBerechtigung funktion) {
		if (benutzer == null) {
			return hasBerechtigung((String) null, funktion);
		}
		return hasBerechtigung(benutzer.getPid(), funktion);
	}

	private void init(final ClientDavInterface davVerbindung) {
		if (davVerbindung == null) {
			reset();
		} else {
			benutzerVerwaltung.init(rahmenwerk);
			final SystemObject oberflaechenObject = rahmenwerk
					.getOberflaechenObject();
			final DataDescription dataDescription = new DataDescription(
					davVerbindung.getDataModel().getAttributeGroup(
							"atg.oberflchenBerechtigung"), davVerbindung
							.getDataModel().getAspect("asp.parameterSoll"));
			davVerbindung.subscribeReceiver(aktualisierer, oberflaechenObject,
					dataDescription, ReceiveOptions.normal(),
					ReceiverRole.receiver());
		}
	}

	private void loadExtensions() {
		final IExtensionRegistry reg = Platform.getExtensionRegistry();
		final IConfigurationElement[] funktionConfigs = reg
				.getConfigurationElementsFor("de.bsvrz.buv.rw.basislib.funktionmitberechtigung");
		for (final IConfigurationElement config : funktionConfigs) {
			final String id = config.getAttribute("id");
			final String bez = config.getAttribute("bezeichnung");
			final String desc = config.getAttribute("beschreibung");
			final String cat = config.getAttribute("category");
			if ((id != null) && (bez != null)) {
				addOberflaechenFunktion(new FunktionMitBerechtigung(config
						.getContributor().getName(), cat, id, bez, desc));
			}
		}
	}

	/**
	 * entfernt einen Listener, der ber nderungen des Parameterdatensatzes
	 * informiert wurde.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removeDatenSatzListener(final PropertyChangeListener listener) {
		propSupport.removePropertyChangeListener(listener);
	}

	@Override
	public void removeListener(final IBerechtigungListener listener) {
		listenerManager.remove(listener);
	}

	@Override
	public void removeOberflaechenFunktion(final String id) {
		synchronized (funktionen) {
			funktionen.remove(id);
			nichtZugeordneteFunktionen.removeFunktion(id);
			for (final PluginMitBerechtigungen plugin : plugins.values()) {
				plugin.removeFunktion(id);
			}
		}
	}

	private void reset() {
		benutzerVerwaltung.reset();
		aktualisiereBerechtigungen(null);
	}

	/**
	 * entfernt eine Instanz des Service "Rahmenwerk" von der Komponente.
	 * 
	 * @param oldRahmenwerk
	 *            der Service
	 */
	protected void unbindRahmenwerk(final Rahmenwerk oldRahmenwerk) {
		this.rahmenwerk = null;
	}

	/**
	 * prft, ob fr de mit dem Systemobjekt bergebene Klasse eine Freigabe fr
	 * die angegebene Funktion besteht.
	 * 
	 * @param klasse
	 *            die Klasse
	 * @param funktion
	 *            die Funktion
	 * @return der Freigabestatus
	 */
	public boolean hasKlasseBerechtigung(final SystemObject klasse,
			final FunktionMitBerechtigung funktion) {

		if (berechtigungsKonzeptAktiv) {
			final BerechtigungsKey key = new BerechtigungsKey(
					new BerechtigungsKlasse(klasse), new Funktion(funktion));
			synchronized (parameterMutex) {
				return berechtigungsParameter.isFreigegeben(key);
			}
		}

		return true;
	}

	/**
	 * ermittelt, ob das Berechtigungskonzept aktiviert ist.
	 * 
	 * @return der Zustand
	 */
	public boolean isBerechtigungsKonzeptAktiv() {
		return berechtigungsKonzeptAktiv;
	}

}
