/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;

/**
 * Reprsentiert eines Plugin in den Contentprovidern zur Darstellung der
 * aktuellen Berechtigungen und im Berechtigungseditor.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: PluginMitBerechtigungen.java 49945 2014-04-02 12:44:01Z peuker $
 */
public class PluginMitBerechtigungen {

	/** Platzhalter-Plugin ID fr Funktionseintrge mit unbekanntem Plug-in. */
	private static final String PLUGIN_NICHT_BEKANNT = "Plugin nicht bekannt";

	private final String id;
	private final Map<String, Funktion> funktionen = new HashMap<String, Funktion>();
	private String bundleName;

	/**
	 * Standardkonstruktor, erstellt eine virtuelle Plugin-Instanz in die alle
	 * nicht zuordenbaren Funktionen eingeordnet werden.
	 */
	public PluginMitBerechtigungen() {
		id = PLUGIN_NICHT_BEKANNT;
	}

	/**
	 * Konstruktor, erzeugt eine Plugin-Reprsentation fr das Plugin mit der
	 * bergebenen Id.
	 * 
	 * @param id
	 *            die Id
	 */
	public PluginMitBerechtigungen(final String id) {
		this.id = id;

		final Bundle bundle = Platform.getBundle(id);
		if (bundle != null) {
			bundleName = bundle.getHeaders().get("Bundle-Name");
		} else {
			bundleName = null;
		}
	}

	/**
	 * liefert die Id des Plugins.
	 * 
	 * @return die Id
	 */
	public String getId() {
		return id;
	}

	/**
	 * ermittelt, um es sich um ein registriertes Plug-in oder den Container fr
	 * nicht zuordenbare Funktionen handelt.
	 * 
	 * @return der Zustand
	 */
	public boolean isPlugin() {
		return !PLUGIN_NICHT_BEKANNT.equals(id);
	}

	/**
	 * fgt dem Plugin eine Funktionsdefinition hinzu.
	 * 
	 * @param funktion
	 *            die Funktion
	 */
	public void addFunktion(final Funktion funktion) {
		synchronized (funktionen) {
			funktionen.put(funktion.getId(), funktion);
		}
	}

	/**
	 * entfernt eine Funktion von der Plugin-Reprsentation.
	 * 
	 * @param funktionsId
	 *            die Id der Funktion
	 */
	public void removeFunktion(final String funktionsId) {
		synchronized (funktionen) {
			funktionen.remove(funktionsId);
		}
	}

	/**
	 * liefert alle Funktion, die dem Plugin zugeordnet sind.
	 * 
	 * @return die Funktionen
	 */
	public Collection<Funktion> getFunktionen() {
		return funktionen.values();
	}

	/**
	 * liefert den Name des Bundles oder die ID, wenn dieser nicht verfgbar
	 * ist.
	 * 
	 * @return den Name oder die ID.
	 */
	public String getNameOrId() {
		if (bundleName == null) {
			return id;
		}

		return bundleName;
	}

	@Override
	public String toString() {
		if (bundleName != null) {
			return bundleName + " (" + id + ")";
		}
		return id;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final PluginMitBerechtigungen other = (PluginMitBerechtigungen) obj;
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		return true;
	}
}
