/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import java.io.IOException;
import java.io.StringReader;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.jface.preference.IPreferenceStore;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungAvailabilityListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungChangeListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungLocation;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungOwnerType;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsFactory;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Manager zur Verwaltung der verschiedenen Einstellungsspeicher des Systems.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungenSpeichermanager.java 46953 2013-11-05 10:05:57Z
 *          peuker $
 */
public class EinstellungenSpeichermanager {

	private final EinstellungsListenerManager listenerManager = new EinstellungsListenerManager();
	private final Map<SpeicherKey, EinstellungsSpeicher> speicherVerwaltung = new HashMap<SpeicherKey, EinstellungsSpeicher>();

	private final Object connectMutex = new Object();
	private final Object saveMutex = new Object();
	private boolean verbunden;
	private Rahmenwerk rahmenwerk;

	private EinstellungenImpl einstellungen;

	private DavConnector davConnector;
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor.
	 *
	 * @param einstellungen
	 *            der Einstellungen-Service
	 * @param rahmenwerk
	 *            das verwendete {@link Rahmenwerk}
	 */
	public EinstellungenSpeichermanager(final EinstellungenImpl einstellungen,
			final Rahmenwerk rahmenwerk) {
		this.einstellungen = einstellungen;
		this.rahmenwerk = rahmenwerk;
		this.rahmenwerk.addDavVerbindungsListener(new DavVerbindungsListener() {

			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				disconnect();
			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				return false;
			}

			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				connect(event.getDavVerbindung());
			}
		});

		connect(rahmenwerk.getDavVerbindung());
		intialisiereLokalenSpeicher();
	}

	/**
	 * fgt einen Listener hinzu, der ber nderungen der Verfgbarkeit des
	 * Einstellungsspeichers benachrichtigt wird.
	 *
	 * @param listener
	 *            der Listener
	 */
	public void addAvailabilityListener(
			final EinstellungAvailabilityListener listener) {
		listenerManager.add(listener);
	}

	/**
	 * fgt einen Listener hinzu, der ber nderungen von Einstellungen der
	 * angegebenen Kategorie benachrichtigt wird.
	 *
	 * @param listener
	 *            der Listener
	 * @param category
	 *            die Kategorie
	 */
	public void addChangeListener(final EinstellungChangeListener listener,
			final String category) {
		listenerManager.add(listener, category);
	}

	void connect(final ClientDavInterface davVerbindung) {
		if (davVerbindung != null) {
			synchronized (connectMutex) {
				if (!verbunden) {
					verbunden = true;
					davConnector = new DavConnector(this, rahmenwerk);
					initialisiereLokalenBenutzerSpeicher(rahmenwerk
							.getBenutzer());
				}
			}
		}
	}

	void disconnect() {
		synchronized (connectMutex) {
			if (verbunden) {

				synchronized (speicherVerwaltung) {
					final Set<SpeicherKey> keySet = new HashSet<SpeicherKey>(
							speicherVerwaltung.keySet());
					for (final SpeicherKey key : keySet) {
						if (key.isLocal()
								&& (key.getOwnerType() != EinstellungOwnerType.SYSTEM)) {
							speicherVerwaltung.remove(key);
						}
					}
				}
				verbunden = false;
				listenerManager.fireAvailability(false);
			}
		}
	}

	/**
	 * liefert die Klasse zur Kommunikation des Einstellungsspeichers mit dem
	 * Datenverteilersystem.
	 *
	 * @return der Datenverteiler-Konnektor
	 */
	public DavConnector getDavConnector() {
		return davConnector;
	}

	/**
	 * liefert den verwendeten Rahmenwerk-Service.
	 *
	 * @return den Service
	 */
	public Rahmenwerk getRahmenwerk() {
		return rahmenwerk;
	}

	/**
	 * liefert den unter der angegebenen Adresse abgelegten Einstellungswert
	 * oder <code>null</code>, wenn keiner existiert.
	 *
	 * @param adresse
	 *            die Adresse
	 * @return den Wert oder <code>null</code>
	 * @throws IOException
	 *             der Wert konnte nicht eingelesen werden
	 */
	public Object getValue(final EinstellungsAdresse adresse)
			throws IOException {

		final SpeicherKey key = new SpeicherKey(adresse);
		final EinstellungsSpeicher speicher = speicherVerwaltung.get(key);
		if (speicher == null) {
			return null;
		}
		return speicher.getValue(adresse);
	}

	/**
	 * liefert die Zuordnungsliste der Einstellungen fr den mit dem angegebenen
	 * Speicherkey referenzierten Einstellungsspeicher.
	 *
	 * @param key
	 *            der Schlssel
	 * @return die Zuordnung (Id, Typ)
	 */
	public Map<String, String> getEinstellungsId(final SpeicherKey key) {
		final EinstellungsSpeicher speicher = speicherVerwaltung.get(key);
		if (speicher == null) {
			return Collections.emptyMap();
		}
		return speicher.getEinstellungsId();
	}

	private void initialisiereLokalenBenutzerSpeicher(
			final SystemObject benutzer) {

		if (benutzer == null) {
			return;
		}

		if (rahmenwerk.isOnline()) {
			final IPreferenceStore preferenceStore = RwEinstellungenActivator
					.getDefault().getPreferenceStore();

			final String einstellungsDaten = preferenceStore
					.getString(LokalSpeicher.getPreferenceKey(benutzer.getPid()));
			if (!einstellungsDaten.isEmpty()) {
				parseLokaleEinstellungsdaten(einstellungsDaten,
						new SpeicherKey(EinstellungOwnerType.BENUTZER,
								EinstellungLocation.LOKAL, benutzer.getPid()));
			}
		}
	}

	private void intialisiereLokalenSpeicher() {
		final IPreferenceStore preferenceStore = RwEinstellungenActivator
				.getDefault().getPreferenceStore();

		final String einstellungsDaten = preferenceStore
				.getString(LokalSpeicher.getPreferenceKey(null));

		final SpeicherKey key = new SpeicherKey(EinstellungOwnerType.SYSTEM,
				EinstellungLocation.LOKAL);
		final EinstellungsSpeicher speicher = EinstellungsSpeicher
				.createSpeicher(key, this);
		speicherVerwaltung.put(key, speicher);
		if (!einstellungsDaten.isEmpty()) {
			parseLokaleEinstellungsdaten(einstellungsDaten, key);
		}

		if (rahmenwerk.isOnline()) {
			initialisiereLokalenBenutzerSpeicher(rahmenwerk.getBenutzer());
		}
	}

	private void parseLokaleEinstellungsdaten(final String einstellungsDaten,
			final SpeicherKey speicherKey) {
		try {
			final SAXParserFactory factory = SAXParserFactory.newInstance();
			final SAXParser parser = factory.newSAXParser();
			parser.parse(new InputSource(new StringReader(einstellungsDaten)),
					new EinstellungenXmlHandler(this, speicherKey));
		} catch (final ParserConfigurationException e) {
			EinstellungenSpeichermanager.LOGGER.error(e.getLocalizedMessage());
		} catch (final SAXException e) {
			EinstellungenSpeichermanager.LOGGER.error(e.getLocalizedMessage());
		} catch (final IOException e) {
			EinstellungenSpeichermanager.LOGGER.error(e.getLocalizedMessage());
		}
	}

	/**
	 * entfernt einen Listener, der ber nderungen der Verfgbarkeit des
	 * Einstellungsspeichers benachrichtigt wird.
	 *
	 * @param listener
	 *            der Listener
	 */
	public void removeAvailabilityListener(
			final EinstellungAvailabilityListener listener) {
		listenerManager.remove(listener);
	}

	/**
	 * entfernt einen Listener, der ber nderungen von Einstellungen der
	 * angegebenen Kategorie benachrichtigt wird.
	 *
	 * @param listener
	 *            der Listener
	 * @param category
	 *            die Kategorie
	 */
	public void removeChangeListener(final EinstellungChangeListener listener,
			final String category) {
		listenerManager.remove(listener, category);
	}

	void setEinstellung(final EinstellungsAdresse adresse, final String data) {

		synchronized (saveMutex) {

			final SpeicherKey key = new SpeicherKey(adresse);
			EinstellungsSpeicher speicher = speicherVerwaltung.get(key);
			if (speicher == null) {
				speicher = EinstellungsSpeicher.createSpeicher(key, this);
				speicherVerwaltung.put(key, speicher);
			}

			final SpeicherErgebnis ergebnis = speicher.setValue(adresse, data);

			switch (ergebnis) {
			case CHANGED:
			case NEW:
			case REMOVED:
				try {
					final Object einstellung = speicher.getValue(adresse);
					listenerManager.fireChanges(ergebnis, adresse, einstellung);
				} catch (final IOException e) {
					EinstellungenSpeichermanager.LOGGER.warning(e
							.getLocalizedMessage());
				}
				break;
			case UNCHANGED:
			default:
				break;
			}
		}
	}

	void setValue(final EinstellungsAdresse adresse, final Object einstellung,
			final UrlasserInfo info) throws IOException, EinstellungsSpeicherException {

		synchronized (saveMutex) {

			final EinstellungsFactory factory = validateAccess(adresse);

			final SpeicherKey key = new SpeicherKey(adresse);
			EinstellungsSpeicher speicher = speicherVerwaltung.get(key);
			if (speicher == null) {
				if (key.isLocal()
						&& (key.getOwnerType() != EinstellungOwnerType.SYSTEM)) {
					initialisiereLokalenBenutzerSpeicher(rahmenwerk
							.getDavVerbindung().getDataModel()
							.getObject(key.getPid()));
				}
				speicher = speicherVerwaltung.get(key);
				if (rahmenwerk.isOnline()) {
					if (speicher == null) {
						speicher = EinstellungsSpeicher.createSpeicher(key,
								this);
						speicherVerwaltung.put(key, speicher);
					}
				}

				if (speicher == null) {
					throw new IOException(
							"Einstellungsspeicher ist im Offline-Betrieb nicht verfgbar!");
				}
			}

			SpeicherErgebnis ergebnis;
			if (einstellung == null) {
				ergebnis = speicher.setValue(adresse, null);
			} else {
				ergebnis = speicher.setValue(adresse,
						factory.serialisiere(einstellung));
			}

			switch (ergebnis) {
			case CHANGED:
			case NEW:
			case REMOVED:
				speicher.save(info);
				// if (adresse.getLocation() == EinstellungLocation.LOKAL) {
				listenerManager.fireChanges(ergebnis, adresse, einstellung);
				// }
				saveMutex.notifyAll();
				break;
			case UNCHANGED:
			default:
				break;
			}
		}
	}

	private EinstellungsFactory validateAccess(final EinstellungsAdresse adresse) {
		if (!verbunden) {
			if ((adresse.getLocation() == EinstellungLocation.NETZWERKWEIT)
					|| (adresse.getOwnerType() != EinstellungOwnerType.SYSTEM)) {
				throw new IllegalStateException(
						"Es besteht keine Datenverteilerverbindung");
			}
		}

		final EinstellungsFactory factory = einstellungen.getFactory(adresse
				.getTyp());
		if (factory == null) {
			throw new IllegalStateException(
					"Es ist keine Factory fr die Kategorie: "
							+ adresse.getTyp() + " registriert!");
		}

		return factory;
	}

}
