/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.bsvrz.buv.rw.basislib.einstellungen.Einstellung;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungOwnerType;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenBezeichnerXml;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;

/**
 * Abstrakte Implementierung eines Einstellungsspeichers.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungsSpeicher.java 51069 2014-07-30 13:31:37Z peuker $
 */
public abstract class EinstellungsSpeicher {

	/** Schlssel fr Einstellungen im Einstellungsspeicher. */
	private static class EinstellungenKey {

		private final String typ;
		private final String id;

		public EinstellungenKey(final EinstellungsAdresse adresse) {
			this.typ = adresse.getTyp();
			this.id = adresse.getId();
		}

		@Override
		public String toString() {
			return typ + ": " + id;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = (prime * result) + ((id == null) ? 0 : id.hashCode());
			result = (prime * result) + ((typ == null) ? 0 : typ.hashCode());
			return result;
		}

		@Override
		public boolean equals(final Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (getClass() != obj.getClass()) {
				return false;
			}
			final EinstellungenKey other = (EinstellungenKey) obj;
			if (id == null) {
				if (other.id != null) {
					return false;
				}
			} else if (!id.equals(other.id)) {
				return false;
			}
			if (typ == null) {
				if (other.typ != null) {
					return false;
				}
			} else if (!typ.equals(other.typ)) {
				return false;
			}
			return true;
		}

	}

	static EinstellungsSpeicher createSpeicher(final SpeicherKey key,
			final EinstellungenSpeichermanager manager) {
		EinstellungsSpeicher result;
		switch (key.getLocation()) {
		case LOKAL:
			result = EinstellungsSpeicher.createLokalenSpeicher(
					key.getOwnerType(), key.getPid());
			break;

		case NETZWERKWEIT:
			result = EinstellungsSpeicher.createNetzSpeicher(
					key.getOwnerType(), key.getPid(), manager);
			break;

		default:
			throw new IllegalArgumentException(
					"Unerwartete Einstellungsspeicher-Location: "
							+ key.getLocation());
		}

		return result;
	}

	private static EinstellungsSpeicher createNetzSpeicher(
			final EinstellungOwnerType ownerType, final String pid,
			final EinstellungenSpeichermanager manager) {

		final EinstellungsSpeicher result;

		switch (ownerType) {
		case BENUTZER:
			result = new BenutzerNetzSpeicher(pid, manager);
			break;

		case BENUTZERKLASSE:
			result = new BenutzerKlasseNetzSpeicher(pid, manager);
			break;

		case SYSTEM:
			result = new SystemNetzSpeicher(manager);
			break;

		default:
			throw new IllegalArgumentException(
					"Unerwarteter Einstellungsbesitzer: " + ownerType);
		}

		return result;
	}

	private static EinstellungsSpeicher createLokalenSpeicher(
			final EinstellungOwnerType ownerType, final String pid) {

		final EinstellungsSpeicher result;

		switch (ownerType) {
		case BENUTZER:
		case BENUTZERKLASSE:
			result = new LokalSpeicher(pid);
			break;

		case SYSTEM:
			result = new LokalSpeicher(null);
			break;

		default:
			throw new IllegalArgumentException(
					"Unerwarteter Einstellungsbesitzer: " + ownerType);
		}

		return result;
	}

	private final Map<EinstellungenKey, Einstellung> einstellungen = new HashMap<EinstellungenKey, Einstellung>();

	Object getValue(final EinstellungsAdresse adresse) throws IOException {

		Einstellung einstellung = null;

		synchronized (einstellungen) {
			einstellung = einstellungen.get(new EinstellungenKey(adresse));
		}

		if (einstellung == null) {
			return null;
		}

		return einstellung.getObject();
	}

	SpeicherErgebnis setValue(final EinstellungsAdresse adresse,
			final String daten) {

		SpeicherErgebnis result = SpeicherErgebnis.UNCHANGED;
		final EinstellungenKey key = new EinstellungenKey(adresse);

		synchronized (einstellungen) {
			final Einstellung oldValue = einstellungen.get(key);
			if (oldValue == null) {
				result = SpeicherErgebnis.NEW;
				einstellungen.put(key, new EinstellungImpl(key.typ, daten));
			} else if (!oldValue.getText().equals(daten)) {
				if (daten == null) {
					result = SpeicherErgebnis.REMOVED;
					einstellungen.remove(key);
				} else {
					result = SpeicherErgebnis.CHANGED;
					einstellungen.put(key, new EinstellungImpl(key.typ, daten));
				}
			}
		}

		return result;
	}

	void save(final UrlasserInfo info) throws EinstellungsSpeicherException {

		try {
			final DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			final DocumentBuilder documentBuilder = factory
					.newDocumentBuilder();
			final Document document = documentBuilder.newDocument();
			document.setXmlStandalone(true);

			final Element root = document.createElement("einstellungen");
			document.appendChild(root);

			root.setAttribute("zeitstempel", DateFormat.getDateTimeInstance()
					.format(new Date(System.currentTimeMillis())));

			for (final Entry<EinstellungenKey, Einstellung> einstellung : einstellungen
					.entrySet()) {
				final Element parameterElement = document
						.createElement("parameter");
				parameterElement.setAttribute("id", einstellung.getKey().id);
				parameterElement.setAttribute("typ", einstellung.getKey().typ);
				root.appendChild(parameterElement);

				final CDATASection dataElement = document
						.createCDATASection(einstellung.getValue().getText());
				parameterElement.appendChild(dataElement);
			}

			// write the content into xml file
			final TransformerFactory transformerFactory = TransformerFactory
					.newInstance();
			final Transformer transformer = transformerFactory.newTransformer();
			transformer.setOutputProperty(OutputKeys.ENCODING,
					EinstellungenBezeichnerXml.XML_ENCODING);
			final DOMSource source = new DOMSource(document);

			final ByteArrayOutputStream destinationStream = new ByteArrayOutputStream();
			final StreamResult result = new StreamResult(destinationStream);
			transformer.transform(source, result);

			doSave(new String(destinationStream.toByteArray(),
					EinstellungenBezeichnerXml.XML_ENCODING), info);

		} catch (final DOMException e) {
			throw new EinstellungsSpeicherException(e);
		} catch (final ParserConfigurationException e) {
			throw new EinstellungsSpeicherException(e);
		} catch (final TransformerConfigurationException e) {
			throw new EinstellungsSpeicherException(e);
		} catch (final TransformerException e) {
			throw new EinstellungsSpeicherException(e);
		} catch (final UnsupportedEncodingException e) {
			throw new EinstellungsSpeicherException(e);
		}
	}

	/**
	 * sichert den Inhalt des Einstellungsspeichers.
	 *
	 * @param inhalt
	 *            der Inhalt des Einstellungsspeichers als XML-Struktur
	 * @param info
	 *            die Urlasserinformationen
	 * @throws EinstellungsSpeicherException
	 *             die Daten konnten nicht gesichert werden
	 */
	protected abstract void doSave(String inhalt, UrlasserInfo info)
			throws EinstellungsSpeicherException;

	/**
	 * liefert alle im Einstellungsspeicher vorhandenen Id mit dem zugehrigen
	 * Typ.
	 *
	 * @return die Zuordnungsliste
	 */
	public Map<String, String> getEinstellungsId() {
		final Map<String, String> result = new HashMap<String, String>();
		for (final EinstellungenKey key : einstellungen.keySet()) {
			result.put(key.id, key.typ);
		}
		return result;
	}
}
