/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenBezeichnerXml;

/**
 * Abstrakte Definition einer Menleiste des Rahmenwerks.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: AbstractMenueLeiste.java 45420 2013-08-09 12:12:12Z peuker $
 */
public abstract class AbstractMenueLeiste extends Menue {

	/** Property zur Markierung des bearbeitungsstatus der Defintion. */
	public static final String PROP_DIRTY = "dirty";

	private boolean dirty;
	private final PropertyChangeSupport propSupport = new PropertyChangeSupport(
			this);

	/** Standardkonstruktor, erzeugt eine leere Menleiste. */
	public AbstractMenueLeiste() {
		super();
	}

	/**
	 * Konstruktor.
	 * 
	 * @param xmlStruktur
	 *            die Definition der Menleiste als XML-Struktur
	 * @throws SAXException
	 *             XML-Daten konnten nicht interpretiert werden
	 * @throws IOException
	 *             Daten konnten nicht eingelesen werden
	 * @throws ParserConfigurationException
	 *             fehlerhafte Konfiguration des XML-Parsers
	 */
	public AbstractMenueLeiste(final String xmlStruktur) throws SAXException,
			IOException, ParserConfigurationException {
		final SAXParserFactory factory = SAXParserFactory.newInstance();
		final SAXParser parser = factory.newSAXParser();
		parser.parse(new InputSource(new StringReader(xmlStruktur)),
				new MenuXmlHandler(this));
	}

	/**
	 * liefert eine XML-Struktur, die den Inhalt der Menleiste beschreibt.
	 * 
	 * @return die Struktur
	 * @throws ParserConfigurationException
	 *             fehlerhafte Konfiguration des XML-Parsers
	 * @throws TransformerException
	 *             Daten der Menleiste konnten nicht transformiert werden
	 */
	public synchronized String getXmlFormat()
			throws ParserConfigurationException, TransformerException {
		final DocumentBuilderFactory factory = DocumentBuilderFactory
				.newInstance();
		final DocumentBuilder documentBuilder = factory.newDocumentBuilder();
		final Document document = documentBuilder.newDocument();
		document.setXmlStandalone(true);

		final Element root = document.createElement(getRootTag());
		addXmlNode(document, root);

		for (final MenueElement element : getElemente()) {
			((AbstractMenueElement) element).addXmlNode(document, root);
		}

		// write the content into xml file
		final TransformerFactory transformerFactory = TransformerFactory
				.newInstance();
		final Transformer transformer = transformerFactory.newTransformer();
		transformer.setOutputProperty(OutputKeys.ENCODING,
				EinstellungenBezeichnerXml.XML_ENCODING);
		final DOMSource source = new DOMSource(document);

		final ByteArrayOutputStream destinationStream = new ByteArrayOutputStream();
		final StreamResult result = new StreamResult(destinationStream);
		transformer.transform(source, result);

		return destinationStream.toString();
	}

	/**
	 * liefert den Root-Tag der XML-Struktur, die die Menleiste definiert.
	 * 
	 * @return der Tag
	 */
	protected abstract String getRootTag();

	@Override
	public Menue getMenue() {
		return null;
	}

	@Override
	public void setMenue(final Menue menu) {
		throw new IllegalStateException(
				"Eine Menleiste kann nicht in ein Men eingefgt werden");
	}

	@Override
	public void setDirty() {
		dirty = true;
		propSupport.firePropertyChange(AbstractMenueLeiste.PROP_DIRTY, null,
				dirty);
	}

	/**
	 * ermittelt den Bearbeitungszustand der Definition.
	 * 
	 * @return den Zustand
	 */
	public boolean isDirty() {
		return dirty;
	}

	/** setzt den Bearbeitungszustand der Definition zurck. */
	public void clean() {
		dirty = false;
		propSupport.firePropertyChange(AbstractMenueLeiste.PROP_DIRTY, null,
				dirty);
	}

	@Override
	protected void addXmlNode(final Document document, final Element root) {
		document.appendChild(root);
	}

	/**
	 * liefert den Name der Menleiste.
	 * 
	 * @return den Name
	 */
	public abstract String getName();

	/**
	 * fgt einen Listener hinzu, der ber nderungen der Definition
	 * benachrichtigt werden soll.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addPropertyChangeListener(final PropertyChangeListener listener) {
		propSupport.addPropertyChangeListener(listener);
	}

	/**
	 * entfernt einen Listener , der ber nderungen der Definition
	 * benachrichtigt werden soll.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removePropertyChangeListener(
			final PropertyChangeListener listener) {
		propSupport.removePropertyChangeListener(listener);
	}

	@Override
	public String getVisibleName() {
		return getName();
	}

	@Override
	public void setVisibleName(final String name) {
		// der Name der Menueleiste kann nicht berschrieben werden
	}

	@Override
	protected Menue createMenuCopy() {
		throw new UnsupportedOperationException(
				"Die Erstellung der Kopie einer kompletten Menleiste wird nicht untersttzt!");
	}

	/**
	 * ermittelt, ob die Menleiste leer ist.
	 * 
	 * @return der Zustand
	 */
	public boolean isEmpty() {
		return getElemente().isEmpty();
	}
}
