/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Definition einer Aktion/Funktion innerhalb einer Menleistendefinition.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: MenueAction.java 46077 2013-09-17 19:45:50Z peuker $
 */
public class MenueAction extends NamedMenueElement {

	/** Parameter fr die Aktionsdefinition. */
	public final class Parameter {

		private final String parameterId;
		private final String parameterName;
		private String value;

		private Parameter(final String id, final String name, final String value) {
			this.parameterId = id;
			this.parameterName = name;
			this.value = value;
			if (this.value == null) {
				this.value = "";
			}
		}

		/**
		 * liefert die Id des Parameters.
		 * 
		 * @return die Id
		 */
		public String getParameterId() {
			if ((parameterId == null) || parameterId.isEmpty()) {
				return parameterName;
			}
			return parameterId;
		}

		/**
		 * liefert den Name des Parameters.
		 * 
		 * @return den Name
		 */
		public String getName() {
			return parameterName;
		}

		/**
		 * liefert den Wert des Parameters als Text.
		 * 
		 * @return der Parameterwert
		 */
		public String getValue() {
			return value;
		}

		/**
		 * setzt den Wert des Parameters.
		 * 
		 * @param value
		 *            der Wert
		 */
		public void setValue(final String value) {
			if (value == null) {
				this.value = "";
			} else {
				this.value = value;
			}
			setDirty();
		}
	}

	private Menue menue;
	private final String actionId;
	private String userLabel;
	private final Map<String, Parameter> parameter = new HashMap<String, Parameter>();
	private final String name;
	private final RwToolbarElement vorlage;

	/**
	 * Konstruktor, erzeugt eine Aktionsdefinition auf Basis der bergebenen
	 * Vorlage mit der angegebenen Id und dem angegebenen Name.
	 * 
	 * @param vorlage
	 *            die Vorlage fr die Aktion
	 * @param actionId
	 *            die Id
	 * @param name
	 *            der Name
	 */
	public MenueAction(final RwToolbarElement vorlage, final String actionId,
			final String name) {
		this.vorlage = vorlage;
		this.actionId = actionId;
		this.name = name;
	}

	@Override
	protected void addXmlNode(final Document document, final Element root) {
		final Element element = document.createElement(MenuXmlHandler.TAG_ITEM);
		element.setAttribute(MenuXmlHandler.ATT_ACTION_ID, getActionId());
		element.setAttribute(MenuXmlHandler.ATT_NAME, getName());
		if ((userLabel != null) && (!userLabel.isEmpty())) {
			element.setAttribute(MenuXmlHandler.ATT_USERLABEL, getUserLabel());
		}

		if (!parameter.isEmpty()) {
			final Element parametersNode = document
					.createElement(MenuXmlHandler.TAG_PARAMETERS);
			element.appendChild(parametersNode);

			for (final Parameter item : parameter.values()) {
				final Element parameterNode = document
						.createElement(MenuXmlHandler.TAG_PARAMETER);
				parameterNode.setAttribute(MenuXmlHandler.ATT_ID,
						item.getParameterId());
				parameterNode.setAttribute(MenuXmlHandler.ATT_NAME,
						item.getName());
				final String value = item.getValue();
				if (value != null) {
					parameterNode.setAttribute(MenuXmlHandler.ATT_VALUE, value);
				}
				parametersNode.appendChild(parameterNode);
			}
		}

		root.appendChild(element);
	}

	@Override
	public MenueElement createDeepCopy() {

		final MenueAction copy = new MenueAction(vorlage, actionId, name);
		copy.setUserLabel(userLabel);
		for (final Parameter param : parameter.values()) {
			copy.setParameter(param.getParameterId(), param.getName(),
					param.getValue());
		}

		return copy;
	}

	/**
	 * liefert die Id der Aktion.
	 * 
	 * @return die Id
	 */
	public String getActionId() {
		return actionId;
	}

	/**
	 * liefert die Beschreibung der Aktion (den anzuzeigenden Text).
	 * 
	 * @return den Text
	 */
	public String getDescription() {
		if ((userLabel != null) && !userLabel.isEmpty()) {
			return userLabel;
		}

		return name;
	}

	@Override
	public Menue getMenue() {
		return menue;
	}

	/**
	 * liefert den Standardname der Aktion.
	 * 
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * liefert die fr die Aktion vorgegebenen Parameter.
	 * 
	 * Das Ergebnis ist eine Kombination aus den ber die Vorlage gegebenen und
	 * den durch die konkrete Aktion berschriebenen Parametern.
	 * 
	 * @return die Parameter
	 */
	public Collection<Parameter> getParameters() {
		final Map<String, Parameter> resultMap = new HashMap<String, Parameter>();
		if (vorlage != null) {
			for (final Entry<String, RwToolbarElementParameter> entry : vorlage
					.getParameters().entrySet()) {
				resultMap.put(entry.getKey(), new Parameter(entry.getKey(),
						entry.getValue().getName(), entry.getValue()
								.getDefaultValue()));
			}
		}
		resultMap.putAll(parameter);
		return Collections.unmodifiableCollection(resultMap.values());
	}

	/**
	 * liefert den nutzerdefinierten Name der Aktion oder <code>null</code>,
	 * wenn keiner definiert wurde.
	 * 
	 * @return den Name oder <code>null</code>
	 */
	public String getUserLabel() {
		return userLabel;
	}

	@Override
	public String getVisibleName() {
		final String result = getUserLabel();
		if (result == null) {
			return getName();
		}
		return result;
	}

	private void setDirty() {
		if (menue != null) {
			menue.setDirty();
		}
	}

	@Override
	public void setMenue(final Menue menue) {
		this.menue = menue;
	}

	/**
	 * setzt den Parameter mit den bergebenen Eigenschaften.
	 * 
	 * @param id
	 *            die Id
	 * @param parameterName
	 *            der Name des Parameters
	 * @param value
	 *            der Wert des Parameters
	 */
	public void setParameter(final String id, final String parameterName,
			final String value) {
		parameter.put(id, new Parameter(id, parameterName, value));
	}

	/**
	 * setzt den nutzerdefinierten Name der Aktion.
	 * 
	 * @param userLabel
	 *            der Name
	 */
	public void setUserLabel(final String userLabel) {
		this.userLabel = userLabel;
		setDirty();
	}

	@Override
	public void setVisibleName(final String visibleName) {
		setUserLabel(visibleName);
	}

	@Override
	public String toString() {
		return getDescription();
	}

	/**
	 * ermittelt, ob die Aktion verfgbar ist.
	 * 
	 * Eine Aktion kann nicht verfgbar sein, wenn das Plug-in aus dem die
	 * Aktion ursprnglich stammt nicht (mehr) aktiv ist oder von diesem nicht
	 * mehr bereitgestellt wird.
	 * 
	 * @return den Zustand
	 */
	public boolean isAvailable() {
		return vorlage != null;
	}
}
